/*-------------------------------------------------------------------
Evaluator.c
(c) 2002
Lukas Babicek
xbabice1@fi.muni.cz

Obsahuje funkce pro staticke ohodnoceni stavu na sachovnici
-------------------------------------------------------------------*/

#include<stdio.h>
#include "Evaluator.h"
#include "Chessboard.h"
#include "Game.h"
#include "Search.h"
#include "Moves.h"

//typy

//struktura pescu
typedef struct {
	short empty;      //zda je to prazdne nebo ne
	short open_files; //pocet otevrenych sloupcu
	short white_pawns_file[8];  //pocet bilych pesci na jedn.sloupcich
	short black_pawns_file[8];  //pocet cernych pescu na jednotlivych sloupcich
  short white_pawns_where[8]; //kde se nachazi pesec nejvice dopredu na danem sloupci
  short black_pawns_where[8]; //kde se nachazi pesec nejvice dopredu na danem sloupci
  short white_pawns_on_white; //pocet bilych pescu na bilych polich
  short white_pawns_on_black; //pocet bilych pescu na cernych polich
  short black_pawns_on_white; //pocet cernych pescu na bilych polich
  short black_pawns_on_black; //pocet cernych pescu na cernych polich
} TPawnStructureTableElement, *PTPawnStructureTableElement;

//globalni promenne
int values[8];  //hodnota materialu jednotlivych figur
int material_balance; //material pro bileho minus material pro cerneho
                      //na dane sachovnici - updatuje se v prubehu
                      //(konkretne v ApplyMove, UndoMove)
int strategy_value;   //hodnota, ktera se prida ke staticke ohodnocovaci funkci


/*-------------------------------------------------------------------
void InitValues(void)

nastavi pole hodnot materialu pro figury, material_balance
-------------------------------------------------------------------*/

void InitEvaluator(void)
{
  values[EMPTY]=0;
	values[PAWN]=PAWN_VALUE;
	values[KNIGHT]=KNIGHT_VALUE;
	values[BISHOP]=BISHOP_VALUE;
	values[ROOK]=ROOK_VALUE;
	values[ROOKING]=ROOK_VALUE;
	values[QUEEN]=QUEEN_VALUE;
	values[KING]=KING_VALUE;

	material_balance = 0;
}

/*-------------------------------------------------------------------
void AnalyzePawnStructure(PTPawnStructureTableElement structure)

analyzuje strukturu pescu pro soucasnou sachovnici a vrati
ve structure tuto strukturu
-------------------------------------------------------------------*/

void AnalyzePawnStructure(PTPawnStructureTableElement structure)
{
  int rank,file;
	short pos;
	short open_files=0; //pocet otevrenych sloupcu
  short white_pawns_file=0;
	short black_pawns_file=0;
  short white_pawns_on_white=0; //pocet bilych pescu na bilych polich
  short white_pawns_on_black=0; //pocet bilych pescu na cernych polich
  short black_pawns_on_white=0; //pocet cernych pescu na bilych polich
  short black_pawns_on_black=0; //pocet cernych pescu na cernych polich

 
	for (file=0; file<8; file++){
		structure->white_pawns_where[file] = 0;
		structure->black_pawns_where[file] = 7;
  }
	
	pos = 26;

	for (file=0; file<8; file++) {
    white_pawns_file=0;
 	  black_pawns_file=0;
    for (rank=7; rank>=0; rank--) {
      if (chessboard[pos]==WHITE_PAWN) {
        if (structure->white_pawns_where[file]<rank)
					structure->white_pawns_where[file] = rank;
				white_pawns_file++;
				if ((rank+file) % 2  == 0)
					white_pawns_on_black++;
				else
          white_pawns_on_white++;
			}
      else if (chessboard[pos]==BLACK_PAWN) {
        structure->black_pawns_where[file] = rank;        
				black_pawns_file++;
				if ((rank+file) % 2  == 0)
					black_pawns_on_black++;
				else
          black_pawns_on_white++;
			}
      pos+=12;
		}
		structure->white_pawns_file[file]=white_pawns_file;
		structure->black_pawns_file[file]=black_pawns_file;
    if (white_pawns_file == 0 && black_pawns_file ==0)
			open_files++;
	  pos-=95;
	}

  structure->white_pawns_on_white = white_pawns_on_white;
	structure->white_pawns_on_black = white_pawns_on_black;
	structure->black_pawns_on_white = black_pawns_on_white;
	structure->black_pawns_on_black = black_pawns_on_black;
	structure->open_files = open_files;

}


/*-------------------------------------------------------------------
void EvaluateRooks(void)

Ohodnoti veze - prida ruzne bonusy za umisteni na otevrenych polich,
atd.
structure ... struktura pescu pro danou sachovnici
-------------------------------------------------------------------*/
void EvaluateRooks(PTPawnStructureTableElement structure)
{

	short fig;
	int rank,file,pos;
  int value = 0;

  pos = 26;
	for (rank=7; rank >=0; rank--) {
		for (file=0;file<8;file++) {
      if (chessboard[pos]==WHITE_ROOK) {
          //nejdriv pridelime bonus za vez na 7. radku
					if (rank == 6)
  					value += 5;
				  //je vez na otevrenem/castecne otevrenem sloupci??
					if (structure->white_pawns_file[file]==0) {
						value += 2;
						if (structure->black_pawns_file[file]==0) {
							value += 6;
						}
					}
					//pokud je vez za passed pawnem, zaslouzi si to bonus
					if (structure->white_pawns_file[file] == 1)
						if (file == 0 || structure->black_pawns_where[file-1] <= structure->white_pawns_where[file-1] || structure->black_pawns_file[file-1]==0) 
              if (file == 7 || structure->black_pawns_where[file+1] <= structure->white_pawns_where[file+1] || structure->black_pawns_file[file+1]==0) 
								if (rank < structure->white_pawns_where[file])
									value += 10;

			}
      else if (chessboard[pos]==BLACK_ROOK){
         //nejdriv pridelime bonus za vez na 2. radku
					if (rank == 1)
					  value -= 5;
				 //je vez na otevrenem/castecne otevrenem sloupci??
					if (structure->black_pawns_file[file]==0) {
						value -= 2;
						if (structure->white_pawns_file[file]==0) {
							value -= 6;
						}
					}
					//pokud je vez za passed pawnem, zaslouzi si to bonus
					if (structure->black_pawns_file[file] == 1)
						if (file == 0 || structure->black_pawns_where[file-1] <= structure->white_pawns_where[file-1] || structure->white_pawns_file[file-1]==0) 
              if (file == 7 || structure->black_pawns_where[file+1] <= structure->white_pawns_where[file+1] || structure->white_pawns_file[file+1]==0) 
								if (rank < structure->white_pawns_where[file])
									value -= 10;
				
			}

 
			pos++;

		}
		pos += 4;
	}
	strategy_value += value;

}

/*-------------------------------------------------------------------
void EvaluateDevelopment(PTPawnStructureTableElement structure)

vyhodnoti rozvinuti hry 

structure ... struktura pescu pro danou sachovnici
-------------------------------------------------------------------*/
void EvaluateDevelopment(PTPawnStructureTableElement structure)
{
  short pos,fig, poc, file, rank, col;
	int value = 0;

  //pro bileho hrace
	  //nejprve podporime rozvinuti centralnich pescu
    if (chessboard[101]==WHITE_PAWN)
			value -= 5;
    if (chessboard[102]==WHITE_PAWN)
			value -= 5;
     
    //dale musime spatne ohodnotit kone a strelce na zadnich radcich
		for(pos=110; pos<117; pos++)
			if ((fig=chessboard[pos])==WHITE_KNIGHT || fig==WHITE_BISHOP)
				value -= 5;

    //podporime kone na centralnich ctvercich
    if (chessboard[65] == WHITE_KNIGHT)
			value += 5;
    if (chessboard[66] == WHITE_KNIGHT)
			value += 5;
    if (chessboard[77] == WHITE_KNIGHT)
			value += 5;
    if (chessboard[78] == WHITE_KNIGHT)
			value += 5;

		//podporime strelce na hlav. diagonalach
		for (pos=110; pos >=33; pos-=11)
			if (chessboard[pos]==WHITE_BISHOP)
				value+= 5;
			
		for (pos=117; pos >=26; pos-=13)
			if (chessboard[pos]==WHITE_BISHOP)
				value+= 5;

		//penalizujeme, pokud se kralovna rozvine prilis brzy
		if (chessboard[113]!=WHITE_QUEEN){
  		pos=117;
		  while (chessboard[pos] != WHITE_QUEEN && pos>=26)
  			pos--;
		  if (pos>=26) {
        //spocitame pocet nasich figur na puvodnich mistech
			  poc=0;
  			if (chessboard[110] == WHITE_ROOK)
  				poc++;
			  if (chessboard[111] == WHITE_KNIGHT)
				  poc++;
			  if (chessboard[112] == WHITE_BISHOP)
				  poc++;
			  if (chessboard[114] == WHITE_KING)
				  poc++;
			  if (chessboard[115] == WHITE_BISHOP)
				  poc++;
			  if (chessboard[116] == WHITE_KNIGHT)
				  poc++;
			  if (chessboard[117] == WHITE_ROOK)
				  poc++;
			  value -= poc;
			}
		}

    //penalizujeme "spatne strelce" - pokud je hodne pescu na 
		//polich stejne barvy jako je strelec, je to spatne
    col = BLACK;
		pos = 110;
		for (rank = 0; rank < 7; rank++) {
			for (file = 0; file < 7; file++) {
				if (chessboard[pos] == WHITE_BISHOP){
					if (col == BLACK)
						value -= structure->white_pawns_on_black >> 1;
					else
						value -= structure->white_pawns_on_white >> 1;
				}
				col = -col;
				pos++;
			}
			col = -col;
			pos -= 20;
		}

		//podporime rosadu
		if (white_has_castled)
			value += 10;

  //ted pro cerneho hrace
    //nejprve podporime rozvinuti centralnich pescu
    if (chessboard[41]==BLACK_PAWN)
			value += 5;
    if (chessboard[42]==BLACK_PAWN)
			value += 5;
     
    //dale musime spatne ohodnotit kone a strelce na zadnich radcich
		for(pos=26; pos<33; pos++)
			if ((fig=chessboard[pos])==BLACK_KNIGHT || fig==BLACK_BISHOP)
				value += 5;

    //podporime kone na centralnich ctvercich
    if (chessboard[65] == BLACK_KNIGHT)
			value -= 5;
    if (chessboard[66] == BLACK_KNIGHT)
			value -= 5;
    if (chessboard[77] == BLACK_KNIGHT)
			value -= 5;
    if (chessboard[78] == BLACK_KNIGHT)
			value -= 5;

		//podporime strelce na hlav. diagonalach
		for (pos=110; pos >=33; pos-=11)
			if (chessboard[pos]==BLACK_BISHOP)
				value-= 5;
			
		for (pos=117; pos >=26; pos-=13)
			if (chessboard[pos]==BLACK_BISHOP)
				value-= 5;

		//penalizujeme, pokud se kralovna rozvine prilis brzy
		if (chessboard[29]!=BLACK_QUEEN){
  		pos=26;
		  while (chessboard[pos] != WHITE_QUEEN && pos<=117)
  			pos++;
		  if (pos<=117) {
        //spocitame pocet nasich figur na puvodnich mistech
			  poc=0;
  			if (chessboard[26] == BLACK_ROOK)
  				poc++;
			  if (chessboard[27] == BLACK_KNIGHT)
				  poc++;
			  if (chessboard[28] == BLACK_BISHOP)
				  poc++;
			  if (chessboard[30] == BLACK_KING)
				  poc++;
			  if (chessboard[31] == BLACK_BISHOP)
				  poc++;
			  if (chessboard[32] == BLACK_KNIGHT)
				  poc++;
			  if (chessboard[33] == BLACK_ROOK)
				  poc++;
			  value += poc;
			}
		}

    //penalizujeme "spatne strelce" - pokud je hodne pescu na 
		//polich stejne barvy jako je strelec, je to spatne
    col = WHITE;
		pos = 26;
		for (rank = 0; rank < 7; rank++) {
			for (file = 0; file < 7; file++) {
				if (chessboard[pos] == BLACK_BISHOP){
					if (col == BLACK)
						value += structure->black_pawns_on_black >> 1;
					else
						value += structure->black_pawns_on_white >> 1;
				}
				col = -col;
				pos++;
			}
			col = -col;
			pos += 4;
		}

		//podporime rosadu
		if (black_has_castled)
			value -= 10;

  strategy_value += value;
}


/*-------------------------------------------------------------------
int StaticValue(void)

vrati hodnotu staticke funkce pro sachovnici a pro hrace na tahu
-------------------------------------------------------------------*/
int StaticValue(void)
{
	TPawnStructureTableElement structure;

  AnalyzePawnStructure(&structure);
	strategy_value = 0;
  EvaluateRooks(&structure);	
  EvaluateDevelopment(&structure);	
  return nowonmove*(strategy_value+material_balance);
}

