/*-------------------------------------------------------------------
Graphics.c
(c) 2002
Lukas Babicek
xbabice1@fi.muni.cz

Obsahuje funkce pro praci s grafikou - vykresleni sachovnice a figurek,
nasteveni barev, ...
-------------------------------------------------------------------*/


#include "Graphics.h"
#include "Moves.h"
#include "Winmanip.h"
#include "Chessboard.h"

//konstanty a globalni promenne
GLfloat color_black_piece[]=DEFAULT_COLOR_BLACK_PIECE;  //barva vnitrku cerne figurky
GLfloat color_white_piece[]=DEFAULT_COLOR_WHITE_PIECE;  //barva vnitrku bile figurky
GLfloat color_black_border[]=DEFAULT_COLOR_BLACK_BORDER;//barva okraje cerne figurky
GLfloat color_white_border[]=DEFAULT_COLOR_WHITE_BORDER;//barva okraje bile figurky
GLfloat color_black_field[]=DEFAULT_COLOR_BLACK_FIELD;  //barva cerneho pole sachovnice
GLfloat color_white_field[]=DEFAULT_COLOR_WHITE_FIELD;  //barva bileho pole sachovnice
GLfloat color_board_border[]=DEFAULT_COLOR_BOARD_BORDER;//barva cary obtahujici sachovnici
GLfloat color_board_text[]=DEFAULT_COLOR_BOARD_TEXT;    //barva popisku sachovnice

/*-------------------------------------------------------------------
void DrawPawn(GLfloat x,GLfloat y, GLfloat dx, int color)

namaluje pesce na zadane pozici
x,y .. souradnice leveho horniho rohu policka sachovnice
dx .. sirka policka sachovnice
color .. barva, 0=bila figurka, 1=cerna figurka
-------------------------------------------------------------------*/

void DrawPawn(GLfloat x,GLfloat y, GLfloat dx, int color)
{
  glLineWidth(PIECE_BORDER_LINE_WIDTH);

  //Vykresleni vnitrku figurky
  (color==WHITE)?glColor3fv(color_white_piece):glColor3fv(color_black_piece);
  glBegin(GL_POLYGON);
  glVertex2f(x+0.5*dx,y+dx*0.4);
  glVertex2f(x+0.65*dx,y+dx*0.48);
  glVertex2f(x+0.65*dx,y+dx*0.6);
  glVertex2f(x+0.55*dx,y+dx*0.65);
  glVertex2f(x+0.45*dx,y+dx*0.65);
  glVertex2f(x+0.35*dx,y+dx*0.6);
  glVertex2f(x+0.35*dx,y+dx*0.48);
  glVertex2f(x+0.5*dx,y+dx*0.4);
  glEnd();

  glBegin(GL_POLYGON);
  glVertex2f(x+0.55*dx,y+dx*0.65);
  glVertex2f(x+0.60*dx,y+dx*0.70);
  glVertex2f(x+0.55*dx,y+dx*0.75);
  glVertex2f(x+0.45*dx,y+dx*0.75);
  glVertex2f(x+0.40*dx,y+dx*0.70);
  glVertex2f(x+0.45*dx,y+dx*0.65);
  glVertex2f(x+0.55*dx,y+dx*0.65);
  glEnd();

  glBegin(GL_POLYGON);
  glVertex2f(x+0.55*dx,y+dx*0.75);
  glVertex2f(x+0.75*dx,y+dx*0.90);
  glVertex2f(x+0.75*dx,y+dx*0.95);
  glVertex2f(x+0.25*dx,y+dx*0.95);
  glVertex2f(x+0.25*dx,y+dx*0.90);
  glVertex2f(x+0.45*dx,y+dx*0.75);
  glVertex2f(x+0.55*dx,y+dx*0.75);
  glEnd();


  //Vykresleni okraje figurky
  (color==WHITE)?glColor3fv(color_white_border):glColor3fv(color_black_border);
  glBegin(GL_LINE_LOOP);
  glVertex2f(x+0.5*dx,y+dx*0.4);
  glVertex2f(x+0.65*dx,y+dx*0.48);
  glVertex2f(x+0.65*dx,y+dx*0.6);
  glVertex2f(x+0.55*dx,y+dx*0.65);
  glVertex2f(x+0.60*dx,y+dx*0.70);
  glVertex2f(x+0.55*dx,y+dx*0.75);
  glVertex2f(x+0.75*dx,y+dx*0.90);
  glVertex2f(x+0.75*dx,y+dx*0.95);
  glVertex2f(x+0.25*dx,y+dx*0.95);
  glVertex2f(x+0.25*dx,y+dx*0.90);
  glVertex2f(x+0.45*dx,y+dx*0.75);
  glVertex2f(x+0.40*dx,y+dx*0.70);
  glVertex2f(x+0.45*dx,y+dx*0.65);
  glVertex2f(x+0.35*dx,y+dx*0.6);
  glVertex2f(x+0.35*dx,y+dx*0.48);
  glEnd();
}

/*-------------------------------------------------------------------
void DrawRook(GLfloat x,GLfloat y, GLfloat dx, int color)

namaluje vez na zadane pozici
x,y .. souradnice leveho horniho rohu policka sachovnice
dx .. sirka policka sachovnice
color .. barva, 0=bila figurka, 1=cerna figurka
-------------------------------------------------------------------*/

void DrawRook(GLfloat x,GLfloat y, GLfloat dx, int color)
{
  glLineWidth(PIECE_BORDER_LINE_WIDTH);

  //Vykresleni vnitrku figurky
  (color==WHITE)?glColor3fv(color_white_piece):glColor3fv(color_black_piece);

  glRectf(x+0.25*dx,y+0.2*dx,x+0.35*dx,y+0.3*dx);
  glRectf(x+0.45*dx,y+0.2*dx,x+0.55*dx,y+0.3*dx);
  glRectf(x+0.65*dx,y+0.2*dx,x+0.75*dx,y+0.3*dx);

  glRectf(x+0.25*dx,y+0.3*dx,x+0.75*dx,y+0.5*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.25*dx,y+dx*0.5);
  glVertex2f(x+0.75*dx,y+dx*0.5);
  glVertex2f(x+0.6*dx,y+dx*0.55);
  glVertex2f(x+0.4*dx,y+dx*0.55);
  glVertex2f(x+0.25*dx,y+dx*0.5);
  glEnd();

  glRectf(x+0.4*dx,y+0.55*dx,x+0.6*dx,y+0.65*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.4*dx,y+dx*0.65);
  glVertex2f(x+0.6*dx,y+dx*0.65);
  glVertex2f(x+0.85*dx,y+dx*0.8);
  glVertex2f(x+0.85*dx,y+dx*0.9);
  glVertex2f(x+0.15*dx,y+dx*0.9);
  glVertex2f(x+0.15*dx,y+dx*0.8);
  glVertex2f(x+0.4*dx,y+dx*0.65);
  glEnd();


  //Vykresleni okraje figurky
  (color==WHITE)?glColor3fv(color_white_border):glColor3fv(color_black_border);
  glBegin(GL_LINE_LOOP);
  glVertex2f(x+0.25*dx,y+dx*0.2);
  glVertex2f(x+0.35*dx,y+dx*0.2);
  glVertex2f(x+0.35*dx,y+dx*0.3);
  glVertex2f(x+0.45*dx,y+dx*0.3);
  glVertex2f(x+0.45*dx,y+dx*0.2);
  glVertex2f(x+0.55*dx,y+dx*0.2);
  glVertex2f(x+0.55*dx,y+dx*0.3);
  glVertex2f(x+0.65*dx,y+dx*0.3);
  glVertex2f(x+0.65*dx,y+dx*0.2);
  glVertex2f(x+0.75*dx,y+dx*0.2);
  glVertex2f(x+0.75*dx,y+dx*0.5);
  glVertex2f(x+0.6*dx,y+dx*0.55);
  glVertex2f(x+0.6*dx,y+dx*0.65);
  glVertex2f(x+0.85*dx,y+dx*0.8);
  glVertex2f(x+0.85*dx,y+dx*0.9);
  glVertex2f(x+0.15*dx,y+dx*0.9);
  glVertex2f(x+0.15*dx,y+dx*0.8);
  glVertex2f(x+0.4*dx,y+dx*0.65);
  glVertex2f(x+0.4*dx,y+dx*0.55);
  glVertex2f(x+0.25*dx,y+dx*0.5);
  glEnd();
}

/*-------------------------------------------------------------------
void DrawBishop(GLfloat x,GLfloat y, GLfloat dx, int color)

namaluje strelce na zadane pozici
x,y .. souradnice leveho horniho rohu policka sachovnice
dx .. sirka policka sachovnice
color .. barva, 0=bila figurka, 1=cerna figurka
-------------------------------------------------------------------*/

void DrawBishop(GLfloat x,GLfloat y, GLfloat dx, int color)
{
  glLineWidth(PIECE_BORDER_LINE_WIDTH);

  //Vykresleni vnitrku figurky
  (color==WHITE)?glColor3fv(color_white_piece):glColor3fv(color_black_piece);


  glBegin(GL_POLYGON);
  glVertex2f(x+0.4*dx,y+dx*0.1);
  glVertex2f(x+0.6*dx,y+dx*0.1);
  glVertex2f(x+0.65*dx,y+dx*0.15);
  glVertex2f(x+0.5*dx,y+dx*0.25);
  glVertex2f(x+0.3*dx,y+dx*0.3);
  glVertex2f(x+0.3*dx,y+dx*0.2);
  glVertex2f(x+0.4*dx,y+dx*0.1);
  glEnd();

  glBegin(GL_POLYGON);
  glVertex2f(x+0.5*dx,y+dx*0.25);
  glVertex2f(x+0.7*dx,y+dx*0.2);
  glVertex2f(x+0.7*dx,y+dx*0.3);
  glVertex2f(x+0.6*dx,y+dx*0.4);
  glVertex2f(x+0.4*dx,y+dx*0.4);
  glVertex2f(x+0.5*dx,y+dx*0.25);
  glEnd();

  glBegin(GL_POLYGON);
  glVertex2f(x+0.4*dx,y+dx*0.4);
  glVertex2f(x+0.3*dx,y+dx*0.3);
  glVertex2f(x+0.5*dx,y+dx*0.25);
  glVertex2f(x+0.4*dx,y+dx*0.4);
  glEnd();

  glRectf(x+0.4*dx,y+0.4*dx,x+0.6*dx,y+0.45*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.6*dx,y+dx*0.45);
  glVertex2f(x+0.7*dx,y+dx*0.5);
  glVertex2f(x+0.7*dx,y+dx*0.6);
  glVertex2f(x+0.6*dx,y+dx*0.65);
  glVertex2f(x+0.4*dx,y+dx*0.65);
  glVertex2f(x+0.3*dx,y+dx*0.6);
  glVertex2f(x+0.3*dx,y+dx*0.5);
  glVertex2f(x+0.4*dx,y+dx*0.45);
  glEnd();

  glRectf(x+0.4*dx,y+0.65*dx,x+0.6*dx,y+0.75*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.6*dx,y+dx*0.75);
  glVertex2f(x+0.8*dx,y+dx*0.85);
  glVertex2f(x+0.8*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.75);
  glVertex2f(x+0.6*dx,y+dx*0.75);
  glEnd();


  //Vykresleni okraje figurky
  (color==WHITE)?glColor3fv(color_white_border):glColor3fv(color_black_border);

  glBegin(GL_LINE_LOOP);
  glVertex2f(x+0.4*dx,y+dx*0.1);
  glVertex2f(x+0.6*dx,y+dx*0.1);
  glVertex2f(x+0.65*dx,y+dx*0.15);
  glVertex2f(x+0.5*dx,y+dx*0.25);
  glVertex2f(x+0.7*dx,y+dx*0.2);
  glVertex2f(x+0.7*dx,y+dx*0.3);
  glVertex2f(x+0.6*dx,y+dx*0.4);
  glVertex2f(x+0.6*dx,y+dx*0.45);
  glVertex2f(x+0.7*dx,y+dx*0.5);
  glVertex2f(x+0.7*dx,y+dx*0.6);
  glVertex2f(x+0.6*dx,y+dx*0.65);
  glVertex2f(x+0.6*dx,y+dx*0.75);
  glVertex2f(x+0.8*dx,y+dx*0.85);
  glVertex2f(x+0.8*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.75);
  glVertex2f(x+0.4*dx,y+dx*0.65);
  glVertex2f(x+0.3*dx,y+dx*0.6);
  glVertex2f(x+0.3*dx,y+dx*0.5);
  glVertex2f(x+0.4*dx,y+dx*0.45);
  glVertex2f(x+0.4*dx,y+dx*0.4);
  glVertex2f(x+0.3*dx,y+dx*0.3);
  glVertex2f(x+0.3*dx,y+dx*0.2);
  glVertex2f(x+0.4*dx,y+dx*0.1);
  glEnd();
}

/*-------------------------------------------------------------------
void DrawKnight(GLfloat x,GLfloat y, GLfloat dx, int color)

namaluje kone na zadane pozici
x,y .. souradnice leveho horniho rohu policka sachovnice
dx .. sirka policka sachovnice
color .. barva, 0=bila figurka, 1=cerna figurka
-------------------------------------------------------------------*/

void DrawKnight(GLfloat x,GLfloat y, GLfloat dx, int color)
{
  glLineWidth(PIECE_BORDER_LINE_WIDTH);

  //Vykresleni vnitrku figurky
  (color==WHITE)?glColor3fv(color_white_piece):glColor3fv(color_black_piece);

  glRectf(x+0.3*dx,y+0.15*dx,x+0.4*dx,y+0.2*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.3*dx,y+dx*0.2);
  glVertex2f(x+0.4*dx,y+dx*0.2);
  glVertex2f(x+0.75*dx,y+dx*0.4);
  glVertex2f(x+0.75*dx,y+dx*0.45);
  glVertex2f(x+0.3*dx,y+dx*0.45);
  glVertex2f(x+0.3*dx,y+dx*0.2);
  glEnd();

  glBegin(GL_POLYGON);
  glVertex2f(x+0.65*dx,y+dx*0.45);
  glVertex2f(x+0.75*dx,y+dx*0.5);
  glVertex2f(x+0.75*dx,y+dx*0.55);
  glVertex2f(x+0.4*dx,y+dx*0.55);
  glVertex2f(x+0.3*dx,y+dx*0.45);
  glVertex2f(x+0.65*dx,y+dx*0.45);
  glEnd();

  glRectf(x+0.4*dx,y+0.55*dx,x+0.6*dx,y+0.70*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.6*dx,y+dx*0.7);
  glVertex2f(x+0.75*dx,y+dx*0.85);
  glVertex2f(x+0.75*dx,y+dx*0.9);
  glVertex2f(x+0.25*dx,y+dx*0.9);
  glVertex2f(x+0.25*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glVertex2f(x+0.6*dx,y+dx*0.7);
  glEnd();


  //Vykresleni okraje figurky
  (color==WHITE)?glColor3fv(color_white_border):glColor3fv(color_black_border);

  glRectf(x+0.47*dx,y+0.35*dx,x+0.53*dx,y+0.41*dx);

  glBegin(GL_LINE_LOOP);
  glVertex2f(x+0.3*dx,y+dx*0.15);
  glVertex2f(x+0.4*dx,y+dx*0.15);
  glVertex2f(x+0.4*dx,y+dx*0.2);
  glVertex2f(x+0.75*dx,y+dx*0.4);
  glVertex2f(x+0.75*dx,y+dx*0.45);
  glVertex2f(x+0.65*dx,y+dx*0.45);
  glVertex2f(x+0.75*dx,y+dx*0.5);
  glVertex2f(x+0.75*dx,y+dx*0.55);
  glVertex2f(x+0.6*dx,y+dx*0.55);
  glVertex2f(x+0.6*dx,y+dx*0.7);
  glVertex2f(x+0.75*dx,y+dx*0.85);
  glVertex2f(x+0.75*dx,y+dx*0.9);
  glVertex2f(x+0.25*dx,y+dx*0.9);
  glVertex2f(x+0.25*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glVertex2f(x+0.4*dx,y+dx*0.55);
  glVertex2f(x+0.3*dx,y+dx*0.45);

  glEnd();
}

/*-------------------------------------------------------------------
void DrawKing(GLfloat x,GLfloat y, GLfloat dx, int color)

namaluje krale na zadane pozici
x,y .. souradnice leveho horniho rohu policka sachovnice
dx .. sirka policka sachovnice
color .. barva, 0=bila figurka, 1=cerna figurka
-------------------------------------------------------------------*/

void DrawKing(GLfloat x,GLfloat y, GLfloat dx, int color)
{
  glLineWidth(PIECE_BORDER_LINE_WIDTH);

  //Vykresleni vnitrku figurky
  (color==WHITE)?glColor3fv(color_white_piece):glColor3fv(color_black_piece);

  glRectf(x+0.45*dx,y+0.05*dx,x+0.55*dx,y+0.3*dx);
  glRectf(x+0.35*dx,y+0.15*dx,x+0.65*dx,y+0.25*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.45*dx,y+dx*0.3);
  glVertex2f(x+0.55*dx,y+dx*0.3);
  glVertex2f(x+0.7*dx,y+dx*0.4);
  glVertex2f(x+0.7*dx,y+dx*0.5);
  glVertex2f(x+0.6*dx,y+dx*0.6);
  glVertex2f(x+0.4*dx,y+dx*0.6);
  glVertex2f(x+0.3*dx,y+dx*0.5);
  glVertex2f(x+0.3*dx,y+dx*0.4);
  glVertex2f(x+0.45*dx,y+dx*0.3);
  glEnd();

  glRectf(x+0.4*dx,y+0.6*dx,x+0.6*dx,y+0.7*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glVertex2f(x+0.6*dx,y+dx*0.7);
  glVertex2f(x+0.8*dx,y+dx*0.85);
  glVertex2f(x+0.8*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glEnd();


  //Vykresleni okraje figurky
  (color==WHITE)?glColor3fv(color_white_border):glColor3fv(color_black_border);


  glBegin(GL_LINE_LOOP);
  glVertex2f(x+0.45*dx,y+dx*0.05);
  glVertex2f(x+0.55*dx,y+dx*0.05);
  glVertex2f(x+0.55*dx,y+dx*0.15);
  glVertex2f(x+0.65*dx,y+dx*0.15);
  glVertex2f(x+0.65*dx,y+dx*0.25);
  glVertex2f(x+0.55*dx,y+dx*0.25);
  glVertex2f(x+0.55*dx,y+dx*0.3);
  glVertex2f(x+0.7*dx,y+dx*0.4);
  glVertex2f(x+0.7*dx,y+dx*0.5);
  glVertex2f(x+0.6*dx,y+dx*0.6);
  glVertex2f(x+0.6*dx,y+dx*0.7);
  glVertex2f(x+0.8*dx,y+dx*0.85);
  glVertex2f(x+0.8*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glVertex2f(x+0.4*dx,y+dx*0.6);
  glVertex2f(x+0.3*dx,y+dx*0.5);
  glVertex2f(x+0.3*dx,y+dx*0.4);
  glVertex2f(x+0.45*dx,y+dx*0.3);
  glVertex2f(x+0.45*dx,y+dx*0.25);
  glVertex2f(x+0.35*dx,y+dx*0.25);
  glVertex2f(x+0.35*dx,y+dx*0.15);
  glVertex2f(x+0.45*dx,y+dx*0.15);
  glEnd();
}

/*-------------------------------------------------------------------
void DrawQueen(GLfloat x,GLfloat y, GLfloat dx, int color)

namaluje kralovnu na zadane pozici
x,y .. souradnice leveho horniho rohu policka sachovnice
dx .. sirka policka sachovnice
color .. barva, 0=bila figurka, 1=cerna figurka
-------------------------------------------------------------------*/

void DrawQueen(GLfloat x,GLfloat y, GLfloat dx, int color)
{
  glLineWidth(PIECE_BORDER_LINE_WIDTH);

  //Vykresleni vnitrku figurky
  (color==WHITE)?glColor3fv(color_white_piece):glColor3fv(color_black_piece);

  glBegin(GL_TRIANGLES);
  glVertex2f(x+0.2*dx,y+dx*0.05);
  glVertex2f(x+0.4*dx,y+dx*0.2);
  glVertex2f(x+0.2*dx,y+dx*0.2);
  glVertex2f(x+0.4*dx,y+dx*0.2);
  glVertex2f(x+0.5*dx,y+dx*0.05);
  glVertex2f(x+0.6*dx,y+dx*0.2);
  glVertex2f(x+0.6*dx,y+dx*0.2);
  glVertex2f(x+0.8*dx,y+dx*0.05);
  glVertex2f(x+0.8*dx,y+dx*0.2);
  glEnd();

  glBegin(GL_POLYGON);
  glVertex2f(x+0.2*dx,y+dx*0.2);
  glVertex2f(x+0.8*dx,y+dx*0.2);
  glVertex2f(x+0.6*dx,y+dx*0.3);
  glVertex2f(x+0.4*dx,y+dx*0.3);
  glVertex2f(x+0.2*dx,y+dx*0.2);
  glEnd();

  glBegin(GL_POLYGON);
  glVertex2f(x+0.4*dx,y+dx*0.3);
  glVertex2f(x+0.6*dx,y+dx*0.3);
  glVertex2f(x+0.7*dx,y+dx*0.4);
  glVertex2f(x+0.7*dx,y+dx*0.5);
  glVertex2f(x+0.6*dx,y+dx*0.6);
  glVertex2f(x+0.4*dx,y+dx*0.6);
  glVertex2f(x+0.3*dx,y+dx*0.5);
  glVertex2f(x+0.3*dx,y+dx*0.4);
  glVertex2f(x+0.4*dx,y+dx*0.3);
  glEnd();

  glRectf(x+0.4*dx,y+0.6*dx,x+0.6*dx,y+0.7*dx);

  glBegin(GL_POLYGON);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glVertex2f(x+0.6*dx,y+dx*0.7);
  glVertex2f(x+0.8*dx,y+dx*0.85);
  glVertex2f(x+0.8*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glEnd();


  //Vykresleni okraje figurky
  (color==WHITE)?glColor3fv(color_white_border):glColor3fv(color_black_border);


  glBegin(GL_LINE_LOOP);
  glVertex2f(x+0.2*dx,y+dx*0.05);
  glVertex2f(x+0.4*dx,y+dx*0.2);
  glVertex2f(x+0.5*dx,y+dx*0.05);
  glVertex2f(x+0.6*dx,y+dx*0.2);
  glVertex2f(x+0.8*dx,y+dx*0.05);
  glVertex2f(x+0.8*dx,y+dx*0.2);
  glVertex2f(x+0.6*dx,y+dx*0.3);
  glVertex2f(x+0.7*dx,y+dx*0.4);
  glVertex2f(x+0.7*dx,y+dx*0.5);
  glVertex2f(x+0.6*dx,y+dx*0.6);
  glVertex2f(x+0.6*dx,y+dx*0.7);
  glVertex2f(x+0.8*dx,y+dx*0.85);
  glVertex2f(x+0.8*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.9);
  glVertex2f(x+0.2*dx,y+dx*0.85);
  glVertex2f(x+0.4*dx,y+dx*0.7);
  glVertex2f(x+0.4*dx,y+dx*0.6);
  glVertex2f(x+0.3*dx,y+dx*0.5);
  glVertex2f(x+0.3*dx,y+dx*0.4);
  glVertex2f(x+0.4*dx,y+dx*0.3);
  glVertex2f(x+0.2*dx,y+dx*0.2);
  glEnd();
}

/*-------------------------------------------------------------------
void DrawChessboard(int win_width,int win_height)
Namaluje sachovnici 8x8
win_width .. sirka okna s sachovnici (je stejna jako vyska okna)
-------------------------------------------------------------------*/

void DrawChessboard(double win_width)
{
  double dx,  //dx je sirka jednoho policka sachovnice
		     x,y;
  int i,j;
	short color,fig;

  dx = (win_width-2*BOARD_BORDER_LINE_WIDTH-BOARD_GAP)/8;

  for(i=0; i<8;i++){
	for(j=0; j<8;j++){
      if ((i+j) % 2 == 0)
   	  glColor3fv(color_white_field);  //bile policko
      else
	     glColor3fv(color_black_field);     //cerne policko

  	   x=i*dx+BOARD_BORDER_LINE_WIDTH+BOARD_GAP;
	   y=j*dx+BOARD_BORDER_LINE_WIDTH;
      glRectd(x,y,x+dx,y+dx);
	 }
  }

  //vykreslime okraj sachovnice
  glColor3fv(color_board_border);
  glRectf(BOARD_GAP,0,BOARD_BORDER_LINE_WIDTH+BOARD_GAP,win_width-BOARD_GAP);
  glRectf(BOARD_GAP,0,win_width,BOARD_BORDER_LINE_WIDTH);
  glRectf(BOARD_GAP,win_width-BOARD_GAP-BOARD_BORDER_LINE_WIDTH,win_width,win_width-BOARD_GAP);
  glRectf(win_width-BOARD_BORDER_LINE_WIDTH,0,win_width,win_width-BOARD_GAP);


  //vykreslime popisky sachovnice
  glColor3fv(color_board_text);
  for (i=0; i<8; i++) {
    glRasterPos2f(BOARD_GAP+BOARD_BORDER_LINE_WIDTH+i*dx+dx/2-5,win_width-5);
    glutBitmapCharacter(DEFAULT_FONT, 'A'+i);
    glRasterPos2f(5,BOARD_BORDER_LINE_WIDTH+(7-i)*dx+dx/2+5);
    glutBitmapCharacter(DEFAULT_FONT, '1'+i);
  }

  //vykreslime figurky
  for (i=0;i<8;i++)
  	for (j=0; j<8;j++){
	    x = BOARD_GAP+BOARD_BORDER_LINE_WIDTH+i*dx;
	    y = BOARD_BORDER_LINE_WIDTH+j*dx;
      fig = chessboard[i+2+(j+2)*12];
      color = (fig > 0)? WHITE: BLACK;
			fig = fig*color;
      switch(fig) {
        case EMPTY:break;
  	    case PAWN:DrawPawn(x,y,dx,color);break;
  		  case ROOK:
				case ROOKING:DrawRook(x,y,dx,color);break;
	      case KNIGHT:DrawKnight(x,y,dx,color);break;
  		  case BISHOP:DrawBishop(x,y,dx,color);break;
	      case QUEEN:DrawQueen(x,y,dx,color);break;
  		 case KING:DrawKing(x,y,dx,color);break;
      }
		}	

  //pokud tahneme nejakou figurku, tak ji vykreslime
  if (isdrag){
  	x = dragfig.x;
	  y = dragfig.y;
	  fig = dragfig.figure;
    color = (fig > 0)? WHITE: BLACK;
  	fig = fig*color;
    switch(fig) {
	    case PAWN:DrawPawn(x,y,dx,color);break;
  		case ROOK:
			case ROOKING:DrawRook(x,y,dx,color);break;
	    case KNIGHT:DrawKnight(x,y,dx,color);break;
  		case BISHOP:DrawBishop(x,y,dx,color);break;
	    case QUEEN:DrawQueen(x,y,dx,color);break;
  		case KING:DrawKing(x,y,dx,color);break;
    }
	}

}


/*-------------------------------------------------------------------
void SetColor(int what, double r, double g, double b);
Nastavi barvu urciteho prvku na sachovnici
what .. barva prvku, ktery ma nastavit
  0..Zvolit barvu bileho pole
  1..Zvolit barvu cerneho pole
  2..Zvolit barvu bilych figurek
  3..Zvolit barvu cernych figurek
  4..Zvolit barvu okraje bilych figurek
  5..Zvolit barvu okraje cernych figurek
  6..Zvolit barvu okraje sachovnice
  7..Zvolit barvu popisku sachovnice

r,g,b .. barevne slozky barvy
-------------------------------------------------------------------*/
void SetColor(int what, double r, double g, double b)
{
  switch(what) {
    case 0:
      color_white_field[0]=r;
      color_white_field[1]=g;
      color_white_field[2]=b;
      break;
    case 1:
      color_black_field[0]=r;
      color_black_field[1]=g;
      color_black_field[2]=b;
      break;
    case 2:
      color_white_piece[0]=r;
      color_white_piece[1]=g;
      color_white_piece[2]=b;
      break;
    case 3:
      color_black_piece[0]=r;
      color_black_piece[1]=g;
      color_black_piece[2]=b;
      break;
    case 4:
      color_white_border[0]=r;
      color_white_border[1]=g;
      color_white_border[2]=b;
      break;
    case 5:
      color_black_border[0]=r;
      color_black_border[1]=g;
      color_black_border[2]=b;
      break;
    case 6:
      color_board_border[0]=r;
      color_board_border[1]=g;
      color_board_border[2]=b;
      break;
    case 7:
      color_board_text[0]=r;
      color_board_text[1]=g;
      color_board_text[2]=b;
      break;

  }
}
