/*-------------------------------------------------------------------
Moves.c
(c) 2002
Lukas Babicek
xbabice1@fi.muni.cz

Obsahuje funkce pro generovani a aplikovani tahu
-------------------------------------------------------------------*/
#include <stdlib.h>
#include <math.h>
#include "Moves.h"
#include "Winmanip.h"
#include "Chessboard.h"
#include "Evaluator.h"
#include "Search.h"
#include "Game.h"

//konstanty a globalni promenne
TMove bestmove;
PTMove start_move,last_move; //prvni a posledni prvek vygenerovaneho seznamu tahu
int nmoves; //pocet vygener. tahu
short white_en_passant;
short black_en_passant;
  //pokud provedl nektery hrac tah s pescem o 2 dopredu, nastavi
  //se do teto promenne pozice tohoto pesce
  //toto se provadi v ApplyMove
  //jinak se to nuluje

//nasledujici promenne urcuji, zda lze provest prislusnou rosadu
short white_castling_kingside;
short white_castling_queenside;
short black_castling_kingside;
short black_castling_queenside;

//promenne urcujici, zda uz byla provedena rosada:
short white_has_castled = 0;
short black_has_castled = 0;


short move_offsets[8][10];
  //velmi dulezite - offsety pro pohyb jednotlivych figur
  //prvni slozka pole znaci figuru
  //druha slozka obsahuje pole ofsetu pro pohyby figury tvaru:
  //0. polozka - pocet tahu (ofsetu)
  //1. polozka - jedna se o dalekonosnou figuru (0..ne,1..ano)
  //dalsi polozky - ofsety

/*-------------------------------------------------------------------
void InitMoveOffsets(void)

Nainicializuje offset pro pohyb figurek + dalsi potrebne veci
-------------------------------------------------------------------*/
void InitMoves(void)
{
  move_offsets[ROOK][0]=4;
  move_offsets[ROOK][1]=1;
	move_offsets[ROOK][2]=-12;
	move_offsets[ROOK][3]=1;
	move_offsets[ROOK][4]=12;
	move_offsets[ROOK][5]=-1;

  move_offsets[ROOKING][0]=4;
  move_offsets[ROOKING][1]=1;
	move_offsets[ROOKING][2]=-12;
	move_offsets[ROOKING][3]=1;
	move_offsets[ROOKING][4]=12;
	move_offsets[ROOKING][5]=-1;

  move_offsets[KNIGHT][0]=8;
  move_offsets[KNIGHT][1]=0;
  move_offsets[KNIGHT][2]=-23;
  move_offsets[KNIGHT][3]=-10;
  move_offsets[KNIGHT][4]=14;
  move_offsets[KNIGHT][5]=25;
  move_offsets[KNIGHT][6]=23;
  move_offsets[KNIGHT][7]=10;
  move_offsets[KNIGHT][8]=-14;
  move_offsets[KNIGHT][9]=-25;

	move_offsets[BISHOP][0]=4;
	move_offsets[BISHOP][1]=1;
	move_offsets[BISHOP][2]=-11;
	move_offsets[BISHOP][3]=13;
	move_offsets[BISHOP][4]=11;
	move_offsets[BISHOP][5]=-13;

	move_offsets[QUEEN][0]=8;
	move_offsets[QUEEN][1]=1;
	move_offsets[QUEEN][2]=-12;
	move_offsets[QUEEN][3]=-11;
	move_offsets[QUEEN][4]=1;
	move_offsets[QUEEN][5]=13;
	move_offsets[QUEEN][6]=12;
	move_offsets[QUEEN][7]=11;
	move_offsets[QUEEN][8]=-1;
	move_offsets[QUEEN][9]=-13;

	move_offsets[KING][0]=8;
	move_offsets[KING][1]=0;
	move_offsets[KING][2]=-12;
	move_offsets[KING][3]=-11;
	move_offsets[KING][4]=1;
	move_offsets[KING][5]=13;
	move_offsets[KING][6]=12;
	move_offsets[KING][7]=11;
	move_offsets[KING][8]=-1;
	move_offsets[KING][9]=-13;

	white_castling_kingside = 1;
	white_castling_queenside = 1;
	black_castling_kingside = 1;
	black_castling_queenside = 1;

  white_has_castled = 0;
  black_has_castled = 0;

  white_en_passant = 0;
  black_en_passant = 0;
}


/*-------------------------------------------------------------------
PTMove AddMove(short from, short to, short type,short what)

vytvori novy tah s danymi atributy a priradi ho na konec seznamu
from,to,type,where  .. atributy tahu
vraci ukazatel na takto pridany tah
-------------------------------------------------------------------*/

PTMove AddMove(short from, short to, short type,short what)
{
  PTMove pom;

	pom = malloc(sizeof(TMove));
	pom->from = from;
	pom->to = to;
	pom->type = type;
	pom->what = what;
	pom->flag = FLAG_NORMAL;  //defaultne tah nezrusi castling, to se nastavi
	                          //az v ApplyMove
  pom->next = NULL;

  
	if (start_move == NULL)
		start_move=last_move=pom;
	else {
		last_move->next = pom;
		last_move = pom;
	}

	nmoves++;
	return pom;
}

/*-------------------------------------------------------------------
void FreeAllMoves(void)

uvolni z pameti seznam tahu
-------------------------------------------------------------------*/

void FreeAllMoves(void)
{
  PTMove pom, pom1;

	pom=start_move;

	while (pom != NULL){
		pom1 = pom;
		pom = pom->next;
		free(pom1);
	}
	start_move = last_move = NULL;
}

/*-------------------------------------------------------------------
int GenerateMoves(void)

vygeneruje vsechny mozne tahy z daneho stavu sachovnice 
(podle promenne nowonmove bud pro cerneho nebo pro bileho)
Vraci pocet takto vygenerovanych tahu
-------------------------------------------------------------------*/

int GenerateMoves(void) {
  short int i,j,k; //ridici promenne cyklu
	short rank, file; //radek a sloupec na sachovnici (1-8)
	short pos; //pozice na sachovnici
	short noffsets; //pocet offsetu pohybu pro danou figuru
	short fig, fig1, fig2,col1,col2,pos1,pom; //nejake pomocne
  short king, rooking;

	start_move = last_move = NULL;
	nmoves = 0;

	pos = 26;
	//zaciname v levem hornim rohu sachovnice
	rank = 8;
	file = 1;

	king = -nowonmove*KING;
	rooking = -nowonmove*ROOKING;

  for (j=2; j<10; j++) {
		for (i=2; i<10;i++) {
      fig = chessboard[pos];
			col1 = (fig>0) ? WHITE: BLACK;
			fig1 = fig / col1;
			if (col1 == nowonmove && fig!=EMPTY) {
			switch(fig) {
			  case EMPTY:break;
        //nejprve osetrime pesce
				case WHITE_PAWN:
          //tah o jedno pole dopredu bez povyseni
					if (chessboard[pos-12]==EMPTY && rank<7) {
					  AddMove(pos,pos-12,MOVE_ORDINARY,EMPTY);
            //pohyb o 2 pole dopredu
					  //v ApplyMove se pak musi nastavit EN-PASSANT!
						if (rank==2 && chessboard[pos-24]==EMPTY) {
              AddMove(pos, pos-24,MOVE_ORDINARY,EMPTY);
            }
					}
					//tah o jedno dopredu s povysenim
					if (chessboard[pos-12]==EMPTY && rank == 7){
						AddMove(pos,pos-12,MOVE_PROMOTION_QUEEN,EMPTY);
						AddMove(pos,pos-12,MOVE_PROMOTION_ROOK,EMPTY);
						AddMove(pos,pos-12,MOVE_PROMOTION_BISHOP,EMPTY);
						AddMove(pos,pos-12,MOVE_PROMOTION_KNIGHT,EMPTY);
          }
					//test na vyhozeni cizi figurky pescem
					if ((pom=chessboard[pos-11])<0 && pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
						  //musime taky osetrit povyseni! 
							if (rank<7)
                AddMove(pos,pos-11,MOVE_CAPTURE,pom);
							else {
    						AddMove(pos,pos-11,MOVE_PROMOTION_QUEEN,pom);
    						AddMove(pos,pos-11,MOVE_PROMOTION_ROOK,pom);
    						AddMove(pos,pos-11,MOVE_PROMOTION_BISHOP,pom);
    						AddMove(pos,pos-11,MOVE_PROMOTION_KNIGHT,pom);
							}
					}
					if ((pom=chessboard[pos-13])<0 && pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
              //musime taky osetrit povyseni! 
							if (rank<7)
                AddMove(pos,pos-13,MOVE_CAPTURE,pom);
							else {
    						AddMove(pos,pos-13,MOVE_PROMOTION_QUEEN,pom);
    						AddMove(pos,pos-13,MOVE_PROMOTION_ROOK,pom);
    						AddMove(pos,pos-13,MOVE_PROMOTION_BISHOP,pom);
    						AddMove(pos,pos-13,MOVE_PROMOTION_KNIGHT,pom);
							}
					}
					//brani mimochodem
					if (black_en_passant){
						if (black_en_passant == pos+1)
						  AddMove(pos,pos-11,MOVE_EN_PASSANT, BLACK_PAWN);
						if (black_en_passant == pos-1)
						  AddMove(pos,pos-13,MOVE_EN_PASSANT, BLACK_PAWN);
					}   
				break;
				case BLACK_PAWN:
					//tah o jedno pole dopredu bez povyseni
          if (chessboard[pos+12]==EMPTY && rank>2) {
              AddMove(pos,pos+12,MOVE_ORDINARY,EMPTY);        
              //pohyb o 2 pole dopredu
	  				  //v ApplyMove se pak musi nastavit EN-PASSANT!
							if (chessboard[pos+24]==EMPTY && rank==7) {
                AddMove(pos,pos+24,MOVE_ORDINARY,EMPTY);
              }
					}
					//tah o jedno dopredu s povysenim
          if (chessboard[pos+12]==EMPTY && rank==2) {					
							AddMove(pos,pos+12,MOVE_PROMOTION_QUEEN,EMPTY);
							AddMove(pos,pos+12,MOVE_PROMOTION_ROOK,EMPTY);
							AddMove(pos,pos+12,MOVE_PROMOTION_BISHOP,EMPTY);
							AddMove(pos,pos+12,MOVE_PROMOTION_KNIGHT,EMPTY);
          }  
					//test na vyhozeni cizi figurky pescem
					if ((pom=chessboard[pos+11])>0 && pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
            //osetreni zda nedojde k povyseni
            if (rank>2)
							AddMove(pos,pos+11,MOVE_CAPTURE,pom);
						else {
							AddMove(pos,pos+11,MOVE_PROMOTION_QUEEN,pom);
							AddMove(pos,pos+11,MOVE_PROMOTION_ROOK,pom);
							AddMove(pos,pos+11,MOVE_PROMOTION_BISHOP,pom);
							AddMove(pos,pos+11,MOVE_PROMOTION_KNIGHT,pom);
						}
					}
					if ((pom=chessboard[pos+13])>0 &&pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
            //osetreni zda nedojde k povyseni
            if (rank>2)
							AddMove(pos,pos+13,MOVE_CAPTURE,pom);
						else {
							AddMove(pos,pos+13,MOVE_PROMOTION_QUEEN,pom);
							AddMove(pos,pos+13,MOVE_PROMOTION_ROOK,pom);
							AddMove(pos,pos+13,MOVE_PROMOTION_BISHOP,pom);
							AddMove(pos,pos+13,MOVE_PROMOTION_KNIGHT,pom);
						}
					}
					//brani mimochodem
				  if (white_en_passant){
					  if (white_en_passant == pos+1)
						  AddMove(pos,pos+13,MOVE_EN_PASSANT, WHITE_PAWN);
						if (white_en_passant == pos-1)
						  AddMove(pos,pos+11,MOVE_EN_PASSANT, WHITE_PAWN);
				}   
				break;
				//nasleduje generovani tahu pro figury
		    default:
					noffsets = move_offsets[fig1][0];
					for (k=0; k<noffsets; k++) {
  				  pos1 = pos;
						//nejprve pro dalekonosne figury
						if (move_offsets[fig1][1]) {
						  pos1 += move_offsets[fig1][k+2];
							fig2 = chessboard[pos1];
							while (fig2 == EMPTY && fig2!=ILLEGAL) {
                AddMove(pos,pos1,MOVE_ORDINARY,EMPTY);
  						  pos1 += move_offsets[fig1][k+2];
    						fig2 = chessboard[pos1];
							}
							if (fig2!=ILLEGAL) {
								col2 = (fig2 > 0) ? WHITE : BLACK;
								if (col1 != col2) {
    						  if (fig2 == king || fig2 == rooking) {
		    					  FreeAllMoves();
				   			    return KING_CHECKED;
									}
                  AddMove(pos,pos1,MOVE_CAPTURE,fig2);
								}
							}
						}
            //pro krale a kone
						else {
						  pos1 += move_offsets[fig1][k+2];
							fig2 = chessboard[pos1];
							if (fig2 != ILLEGAL) {
								if (fig2==EMPTY){
                  AddMove(pos,pos1,MOVE_ORDINARY,EMPTY);
								}
  							else {
									col2 = (fig2 > 0)?WHITE:BLACK;
  								if (col1 != col2) {
      						  if (fig2 == king || fig2 == rooking) {
	  	    					  FreeAllMoves();
		  		   			    return KING_CHECKED;
										}
										AddMove(pos,pos1,MOVE_CAPTURE,fig2);
									}
								}
							}
						} //else
					} //for k
					break;
			} //switch
       
			//ted jeste rosady
			if (fig == BLACK_KING && !black_has_castled) {
				if (black_castling_kingside && 
					  chessboard[31] == EMPTY &&
						chessboard[32] == EMPTY &&
						chessboard[33] == BLACK_ROOK)
					AddMove(pos, pos+2, MOVE_BLACK_CASTLING_KINGSIDE, EMPTY);
				if (black_castling_queenside && 
					  chessboard[27] == EMPTY &&
						chessboard[28] == EMPTY &&
						chessboard[29] == EMPTY &&
						chessboard[26] == BLACK_ROOK)
          AddMove(pos, pos-2, MOVE_BLACK_CASTLING_QUEENSIDE, EMPTY);
      }
			else if (fig == WHITE_KING && !white_has_castled) {
				if (white_castling_kingside && 
					  chessboard[115] == EMPTY &&
						chessboard[116] == EMPTY &&
						chessboard[117] == WHITE_ROOK)
					AddMove(pos, pos+2, MOVE_WHITE_CASTLING_KINGSIDE, EMPTY);
				if (white_castling_queenside && 
					  chessboard[111] == EMPTY &&
						chessboard[112] == EMPTY &&
						chessboard[113] == EMPTY &&
						chessboard[110] == WHITE_ROOK)
          AddMove(pos, pos-2, MOVE_WHITE_CASTLING_QUEENSIDE, EMPTY);
      }

			} //if (col1 == nowonmove)	
			pos++;
			file++;
		} //for i
		pos += 4;
		file = 1;
		rank--;
	} //for j
	return nmoves;
}

/*-------------------------------------------------------------------
int GenerateQuiesenceMoves(void) 

vygeneruje tahy odpovidajici brani, en-passant, povyseni pesce - tedy
tahy, ktere muzou drasticky zmenit material na sachovnici)
Vraci pocet takto vygenerovanych tahu
-------------------------------------------------------------------*/

int GenerateQuiesenceMoves(void) {
  short int i,j,k; //ridici promenne cyklu
	short rank, file; //radek a sloupec na sachovnici (1-8)
	short pos; //pozice na sachovnici
	short noffsets; //pocet offsetu pohybu pro danou figuru
	short fig, fig1, fig2,col1,col2,pos1,pom; //nejake pomocne
  short king, rooking;

	start_move = last_move = NULL;
	nmoves = 0;

	pos = 26;
	//zaciname v levem hornim rohu sachovnice
	rank = 8;
	file = 1;

	king = -nowonmove*KING;
	rooking = -nowonmove*ROOKING;

  for (j=2; j<10; j++) {
		for (i=2; i<10;i++) {
      fig = chessboard[pos];
			col1 = (fig>0) ? WHITE: BLACK;
			fig1 = fig / col1;
			if (col1 == nowonmove && fig!=EMPTY) {
			switch(fig) {
			  case EMPTY:break;
        //nejprve osetrime pesce
				case WHITE_PAWN:
					//tah o jedno dopredu s povysenim
					if (chessboard[pos-12]==EMPTY && rank == 7){
						AddMove(pos,pos-12,MOVE_PROMOTION_QUEEN,EMPTY);
						AddMove(pos,pos-12,MOVE_PROMOTION_ROOK,EMPTY);
						AddMove(pos,pos-12,MOVE_PROMOTION_BISHOP,EMPTY);
						AddMove(pos,pos-12,MOVE_PROMOTION_KNIGHT,EMPTY);
          }
					//test na vyhozeni cizi figurky pescem
					if ((pom=chessboard[pos-11])<0 && pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
						  //musime taky osetrit povyseni! 
							if (rank<7)
                AddMove(pos,pos-11,MOVE_CAPTURE,pom);
							else {
    						AddMove(pos,pos-11,MOVE_PROMOTION_QUEEN,pom);
    						AddMove(pos,pos-11,MOVE_PROMOTION_ROOK,pom);
    						AddMove(pos,pos-11,MOVE_PROMOTION_BISHOP,pom);
    						AddMove(pos,pos-11,MOVE_PROMOTION_KNIGHT,pom);
							}
					}
					if ((pom=chessboard[pos-13])<0 && pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
              //musime taky osetrit povyseni! 
							if (rank<7)
                AddMove(pos,pos-13,MOVE_CAPTURE,pom);
							else {
    						AddMove(pos,pos-13,MOVE_PROMOTION_QUEEN,pom);
    						AddMove(pos,pos-13,MOVE_PROMOTION_ROOK,pom);
    						AddMove(pos,pos-13,MOVE_PROMOTION_BISHOP,pom);
    						AddMove(pos,pos-13,MOVE_PROMOTION_KNIGHT,pom);
							}
					}
					//brani mimochodem
					if (black_en_passant){
						if (black_en_passant == pos+1)
						  AddMove(pos,pos-11,MOVE_EN_PASSANT, BLACK_PAWN);
						if (black_en_passant == pos-1)
						  AddMove(pos,pos-13,MOVE_EN_PASSANT, BLACK_PAWN);
					}   
				break;
				case BLACK_PAWN:
					//tah o jedno dopredu s povysenim
          if (chessboard[pos+12]==EMPTY && rank==2) {					
							AddMove(pos,pos+12,MOVE_PROMOTION_QUEEN,EMPTY);
							AddMove(pos,pos+12,MOVE_PROMOTION_ROOK,EMPTY);
							AddMove(pos,pos+12,MOVE_PROMOTION_BISHOP,EMPTY);
							AddMove(pos,pos+12,MOVE_PROMOTION_KNIGHT,EMPTY);
          }  
					//test na vyhozeni cizi figurky pescem
					if ((pom=chessboard[pos+11])>0 && pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
            //osetreni zda nedojde k povyseni
            if (rank>2)
							AddMove(pos,pos+11,MOVE_CAPTURE,pom);
						else {
							AddMove(pos,pos+11,MOVE_PROMOTION_QUEEN,pom);
							AddMove(pos,pos+11,MOVE_PROMOTION_ROOK,pom);
							AddMove(pos,pos+11,MOVE_PROMOTION_BISHOP,pom);
							AddMove(pos,pos+11,MOVE_PROMOTION_KNIGHT,pom);
						}
					}
					if ((pom=chessboard[pos+13])>0 &&pom!=ILLEGAL) {
						if (pom == king || pom == rooking) {
							FreeAllMoves();
							return KING_CHECKED;
						}
            //osetreni zda nedojde k povyseni
            if (rank>2)
							AddMove(pos,pos+13,MOVE_CAPTURE,pom);
						else {
							AddMove(pos,pos+13,MOVE_PROMOTION_QUEEN,pom);
							AddMove(pos,pos+13,MOVE_PROMOTION_ROOK,pom);
							AddMove(pos,pos+13,MOVE_PROMOTION_BISHOP,pom);
							AddMove(pos,pos+13,MOVE_PROMOTION_KNIGHT,pom);
						}
					}
					//brani mimochodem
				  if (white_en_passant){
					  if (white_en_passant == pos+1)
						  AddMove(pos,pos+13,MOVE_EN_PASSANT, WHITE_PAWN);
						if (white_en_passant == pos-1)
						  AddMove(pos,pos+11,MOVE_EN_PASSANT, WHITE_PAWN);
				}   
				break;
				//nasleduje generovani tahu pro figury
		    default:
					noffsets = move_offsets[fig1][0];
					for (k=0; k<noffsets; k++) {
  				  pos1 = pos;
						//nejprve pro dalekonosne figury
						if (move_offsets[fig1][1]) {
						  pos1 += move_offsets[fig1][k+2];
							fig2 = chessboard[pos1];
							while (fig2 == EMPTY && fig2!=ILLEGAL) {
  						  pos1 += move_offsets[fig1][k+2];
    						fig2 = chessboard[pos1];
							}
							if (fig2!=ILLEGAL) {
								col2 = (fig2 > 0) ? WHITE : BLACK;
								if (col1 != col2) {
    						  if (fig2 == king || fig2 == rooking) {
		    					  FreeAllMoves();
				   			    return KING_CHECKED;
									}
                  AddMove(pos,pos1,MOVE_CAPTURE,fig2);
								}
							}
						}
            //pro krale a kone
						else {
						  pos1 += move_offsets[fig1][k+2];
							fig2 = chessboard[pos1];
							if (fig2 != ILLEGAL && fig2!=EMPTY) {
									col2 = (fig2 > 0)?WHITE:BLACK;
  								if (col1 != col2) {
      						  if (fig2 == king || fig2 == rooking) {
	  	    					  FreeAllMoves();
		  		   			    return KING_CHECKED;
										}
										AddMove(pos,pos1,MOVE_CAPTURE,fig2);
									}
								
							}
						} //else
					} //for k
					break;
			} //switch
       

			} //if (col1 == nowonmove)	
			pos++;
			file++;
		} //for i
		pos += 4;
		file = 1;
		rank--;
	} //for j
	return nmoves;
}


//vycisti extra krale pridane po rosade
void ClearExtraKings(void)
{
	if (nowonmove == WHITE){
		if (chessboard[115] == WHITE_ROOKING) {
			chessboard[115] = WHITE_ROOK;
			chessboard[114] = EMPTY;
		}
		else if(chessboard[113] == WHITE_ROOKING){
			chessboard[113] = WHITE_ROOK;
			chessboard[114] = EMPTY;
		}
	}
	else{
		if (chessboard[31] == BLACK_ROOKING) {
			chessboard[31] = BLACK_ROOK;
			chessboard[30] = EMPTY;
		}
		else if(chessboard[29] == BLACK_ROOKING){
			chessboard[29] = BLACK_ROOK;
			chessboard[30] = EMPTY;
		}
	}
}

//aplikuje dany tah na sachovnici + nastavi priznaky (rosada, en-passant)
void ApplyMove(PTMove move)
{
  short fig, from, to, color;
	int what;

	if (move->from == 0) //neplatny tah
		return;

	fig = chessboard[move->from];
	from = move->from;
	to = move->to;
	what = move->what;
	color = (what > 0) ? WHITE:BLACK;
	what = what / color;
	what = color*values[what]; //what bude reprezentovat mater.hodnotu vyhozene figurky
	//vymazeme priznaky en passant
	white_en_passant = 0;
	black_en_passant = 0;

	//nejprve se mrkneme jestli nemame vycistit extra krale pridane po castlingu
  ClearExtraKings();

	chessboard[from] = EMPTY;

  switch (move->type) {
		case MOVE_ORDINARY:
    	chessboard[to] = fig;
			//nastavime en-passant pokud to jde
			if (fig == WHITE_PAWN && from-to == 24)
				white_en_passant = to;
			else if (fig == BLACK_PAWN && to-from == 24)
				black_en_passant = to;
  	break;
		case MOVE_CAPTURE:
    	chessboard[to] = fig;
      material_balance -= what;
    break;
		case MOVE_PROMOTION_QUEEN:
      material_balance -= what;
			if (fig == BLACK_PAWN){
				chessboard[to] = BLACK_QUEEN;
				material_balance += values[PAWN];
				material_balance -= values[QUEEN];
			}
			else {
				chessboard[to] = WHITE_QUEEN;
				material_balance -= values[PAWN];
				material_balance += values[QUEEN];
			}
		break;
		case MOVE_PROMOTION_KNIGHT:
      material_balance -= what;
			if (fig == BLACK_PAWN){
				material_balance += values[PAWN];
				material_balance -= values[KNIGHT];
				chessboard[to] = BLACK_KNIGHT;
			}
			else {
				material_balance -= values[PAWN];
				material_balance += values[KNIGHT];
				chessboard[to] = WHITE_KNIGHT;
			}
		break;
		case MOVE_PROMOTION_BISHOP:
      material_balance -= what;
			if (fig == BLACK_PAWN){
				material_balance += values[PAWN];
				material_balance -= values[BISHOP];
				chessboard[to] = BLACK_BISHOP;
			}
			else {
				material_balance -= values[PAWN];
				material_balance += values[BISHOP];
				chessboard[to] = WHITE_BISHOP;
			}
		break;
		case MOVE_PROMOTION_ROOK:
      material_balance -= what;
			if (fig == BLACK_PAWN){
				material_balance += values[PAWN];
				material_balance -= values[ROOK];
				chessboard[to] = BLACK_ROOK;
			}
			else {
				material_balance -= values[PAWN];
				material_balance += values[ROOK];
				chessboard[to] = WHITE_ROOK;
			}
		break;
		case MOVE_EN_PASSANT:
  		chessboard[to] = fig;
			material_balance -= what;
			if (fig == WHITE_PAWN){
				chessboard[to+12] = EMPTY;
			}
			else {
				chessboard[to-12] = EMPTY;
			}
		break;
		case MOVE_BLACK_CASTLING_KINGSIDE:
			black_has_castled = 1;
			chessboard[32] = BLACK_KING;
			chessboard[33] = EMPTY;
      //pro detekci nespravneho castlingu pridame navic krale
			chessboard[30] = BLACK_KING;
			chessboard[31] = BLACK_ROOKING;
		break;
		case MOVE_BLACK_CASTLING_QUEENSIDE:
			black_has_castled = 1;
			chessboard[28] = BLACK_KING;
			chessboard[26] = EMPTY;
      //pro detekci nespravneho castlingu pridame navic krale
			chessboard[30] = BLACK_KING;
			chessboard[29] = BLACK_ROOKING;
		break;
		case MOVE_WHITE_CASTLING_KINGSIDE:
			white_has_castled = 1;
			chessboard[116] = WHITE_KING;
			chessboard[117] = EMPTY;
      //pro detekci nespravneho castlingu pridame navic krale
			chessboard[114] = WHITE_KING;
			chessboard[115] = WHITE_ROOKING;
		break;
		case MOVE_WHITE_CASTLING_QUEENSIDE:
			white_has_castled = 1;
			chessboard[112] = WHITE_KING;
			chessboard[110] = EMPTY;
      //pro detekci nespravneho castlingu pridame navic krale
			chessboard[114] = WHITE_KING;
			chessboard[113] = WHITE_ROOKING;
		break;
	}

  //otestujeme, zda tento tah nezrusi castling
	if (white_castling_kingside) {
		//pokud se neco pohlo z rozka, tak to zrusi castling
		if (from == 117){
			white_castling_kingside = 0;
      move->flag |= FLAG_WHITE_CASTLING_KINGSIDE;
		}
    //pokud se pohl kral, zrusi castling
		else if (fig == WHITE_KING){
		  white_castling_kingside = 0;
			move->flag |= FLAG_WHITE_CASTLING_KINGSIDE;
		}
	}
	if (white_castling_queenside) {
		//pokud se neco pohlo z rozka, tak to zrusi castling
		if (from == 110){
			white_castling_queenside = 0;
      move->flag |= FLAG_WHITE_CASTLING_QUEENSIDE;
		}
    //pokud se pohl kral zrusi to castling
		else if (fig == WHITE_KING){
		  white_castling_queenside = 0;
			move->flag |= FLAG_WHITE_CASTLING_QUEENSIDE;
		}
	}
	if (black_castling_kingside) {
		//pokud se neco pohlo z rozka, tak to zrusi castling
		if (from == 33){
			black_castling_kingside = 0;
      move->flag |= FLAG_BLACK_CASTLING_KINGSIDE;
		}
    //pokud se pohl kral, zrusi castling
		else if (fig == BLACK_KING){
		  black_castling_kingside = 0;
			move->flag |= FLAG_BLACK_CASTLING_KINGSIDE;
		}
	}
	if (black_castling_queenside) {
		//pokud se neco pohlo z rozka, tak to zrusi castling
		if (from == 26){
			black_castling_queenside = 0;
      move->flag |= FLAG_BLACK_CASTLING_QUEENSIDE;
		}
    //pokud se pohl kral zrusi to castling
		else if (fig == BLACK_KING){
		  black_castling_queenside = 0;
			move->flag |= FLAG_BLACK_CASTLING_QUEENSIDE;
		}
	}
	nowonmove = -nowonmove;

}

			
//vrati dany tah (vrati sachovnici a priznaky do stavu pred tahem)
void UndoMove(PTMove move)
{
  short fig, from, to, color;
  short from_fig, what_fig, from_pos, to_pos;
	int what;


	what = move->what;
	color = (what > 0) ? WHITE:BLACK;
	what = what / color;
	what = color*values[what]; 

	fig = chessboard[move->to];
	from = move->from;
	to = move->to;

  switch (move->type) {
		case MOVE_PROMOTION_QUEEN:
    	chessboard[to] = move->what;
      material_balance += what;
			if (fig < 0){
				chessboard[from] = BLACK_PAWN;
				material_balance -= values[PAWN];
				material_balance += values[QUEEN];
			}
			else {
  			chessboard[from] = WHITE_PAWN;
				material_balance += values[PAWN];
				material_balance -= values[QUEEN];
			}
		break;
		case MOVE_PROMOTION_ROOK:
    	chessboard[to] = move->what;
      material_balance += what;
			if (fig < 0){
				chessboard[from] = BLACK_PAWN;
				material_balance -= values[PAWN];
				material_balance += values[ROOK];
			}
			else {
  			chessboard[from] = WHITE_PAWN;
				material_balance += values[PAWN];
				material_balance -= values[ROOK];
			}
		break;
		case MOVE_PROMOTION_BISHOP:
    	chessboard[to] = move->what;
      material_balance += what;
			if (fig <  0){
				chessboard[from] = BLACK_PAWN;
				material_balance -= values[PAWN];
				material_balance += values[BISHOP];
			}
			else {
  			chessboard[from] = WHITE_PAWN;
				material_balance += values[PAWN];
				material_balance -= values[BISHOP];
			}
		break;
		case MOVE_PROMOTION_KNIGHT:
    	chessboard[to] = move->what;
      material_balance += what;
			if (fig < 0){
				chessboard[from] = BLACK_PAWN;
				material_balance -= values[PAWN];
				material_balance += values[KNIGHT];
			}
			else {
  			chessboard[from] = WHITE_PAWN;
				material_balance += values[PAWN];
				material_balance -= values[KNIGHT];
			}
		break;
		case MOVE_ORDINARY:
    	chessboard[from] = fig;
    	chessboard[to] = move->what;
		break;
		case MOVE_CAPTURE:
			material_balance += what;
    	chessboard[from] = fig;
    	chessboard[to] = move->what;
    break;
		case MOVE_EN_PASSANT:
			material_balance += what;
  		chessboard[from] = fig;
			chessboard[to] = EMPTY;
			if (fig == WHITE_PAWN){
				chessboard[to+12] = BLACK_PAWN;
				black_en_passant = to+12;		
			}
			else {
				chessboard[to-12] = WHITE_PAWN;
			  white_en_passant = to-12;
			}
		break;
		//a nakonec castlingy
		case MOVE_BLACK_CASTLING_KINGSIDE:
      black_has_castled = 0;
			chessboard[33] = BLACK_ROOK;
			chessboard[32] = EMPTY;
			chessboard[31] = EMPTY;
			chessboard[30] = BLACK_KING;
		break;
		case MOVE_BLACK_CASTLING_QUEENSIDE:
      black_has_castled = 0;
			chessboard[26] = BLACK_ROOK;
			chessboard[28] = EMPTY;
			chessboard[29] = EMPTY;
			chessboard[30] = BLACK_KING;
		break;
		case MOVE_WHITE_CASTLING_KINGSIDE:
      white_has_castled = 0;
			chessboard[117] = WHITE_ROOK;
			chessboard[116] = EMPTY;
			chessboard[115] = EMPTY;
			chessboard[114] = WHITE_KING;
		break;
		case MOVE_WHITE_CASTLING_QUEENSIDE:
      white_has_castled = 0;
			chessboard[110] = WHITE_ROOK;
			chessboard[112] = EMPTY;
			chessboard[113] = EMPTY;
			chessboard[114] = WHITE_KING;
		break;

	}

	//pokud tah zrusil moznost nejakeho castlingu, vratime tuto moznost
	if (move->flag & FLAG_WHITE_CASTLING_KINGSIDE)
		white_castling_kingside = 1;
	if (move->flag & FLAG_WHITE_CASTLING_QUEENSIDE)
		white_castling_queenside = 1;
	if (move->flag & FLAG_BLACK_CASTLING_KINGSIDE)
		black_castling_kingside = 1;
	if (move->flag & FLAG_BLACK_CASTLING_QUEENSIDE)
		black_castling_queenside = 1;

  nowonmove = -nowonmove;
}

			
//zkontroluje, zda je tah legalni, pokud ano, tak ho provede
int ControlMove(short from, short to)
{
  PTMove pom, pom1;
  int nmoves;
	short found = 0;
	nowonmove = onmove;

  ClearExtraKings();
	nowonmove = -nowonmove;
	ClearExtraKings();
	nowonmove = -nowonmove;
	
	nmoves = GenerateMoves();

  pom = start_move;
  while (pom != NULL) {
		if (pom->from == from && pom->to == to) {
			found = 1;
			break;
		}
		pom = pom->next;
	}

  if (pom == NULL){
		FreeAllMoves();
		return 0;
  }

	pom1 = malloc(sizeof(TMove));
	pom1 ->from = pom->from;
	pom1 ->to = pom->to;
	pom1 ->eval = pom->eval;
	pom1 ->flag = pom->flag;
	pom1 ->type = pom->type;
	pom1 ->what = pom->what;

	FreeAllMoves();

  //pokud je tahem povyseni, dame hraci moznost vyberu z figurek
	
	if (pom1->type >= 9 && !choosing_promotion) {
		CreatePromotionWindow();
    choosing_promotion = 1;
		free(pom1);
    return 0;
	}

  if (choosing_promotion) {
	  switch (promotion_figure) {
		  case QUEEN: pom1->type = MOVE_PROMOTION_QUEEN;break;
		  case ROOK: pom1->type = MOVE_PROMOTION_ROOK;break;
		  case BISHOP: pom1->type = MOVE_PROMOTION_BISHOP;break;
		  case KNIGHT: pom1->type = MOVE_PROMOTION_KNIGHT;break;
		}
		promotion_figure = 0;
		choosing_promotion = 0;
	}

	

	ApplyMove(pom1);

  //nasleduje kontrola, zda nam souper nemuze v pristim pultahu vzit
	//krale - to bysme se timto tahem dostali do sachu a byl by tedy
	//nelegalni

  nmoves = GenerateMoves();

	if (nmoves == KING_CHECKED){
		UndoMove(pom1);
		free(pom1);
		FreeAllMoves();
		return 0;
	}
	else {
  
		if (onmove == WHITE) {
		  if (pom1->what != EMPTY)
      	KillBlackPiece(pom1->what);
		}
		else {
		  if (pom1->what != EMPTY)
      	KillWhitePiece(pom1->what);
		}
    
		FreeAllMoves();
		nowonmove = -nowonmove;
		ClearExtraKings();
		free(pom1);
		return 1;
	}
}

//zjisti, zda se jedna o mat nebo pat, nastavi podle toho promennou end
void CheckEnding(void) 
{
  PTMove pom, pom1, pom2;
	short islegal = 0;
  int nmoves;

	
	nowonmove = onmove;

  //nejprve zjistime, zda existuje nejaky legalni tah pro hrace na tahu
	nmoves = GenerateMoves();
  pom1 = pom2 = start_move;
  pom = start_move;
  while (pom && !islegal) {
    ApplyMove(pom);
		nmoves = GenerateMoves();
		FreeAllMoves();
		if (nmoves != KING_CHECKED) islegal = 1;
    UndoMove(pom);   
		pom = pom->next;
	}

	FreeMoves(pom2);

  //pokud neexistuje legalni tah, pak se jedna o sach mat nebo pat - zjistime to
  if (!islegal) {

  	nowonmove = -nowonmove;
		nmoves = GenerateMoves();
		if (nmoves == KING_CHECKED)
			end = 1;  //sach mat
		else
			end = 2;  //pat

		FreeAllMoves();
	}

	if (end == 1){
		if (onmove == WHITE)
			SetStatusText("Konec hry - vyhral cerny.");
		else
			SetStatusText("Konec hry - vyhral bily.");
	}
	else if (end == 2) 
		SetStatusText("Konec hry - pat.");


}