/*-------------------------------------------------------------------
Search.c
(c) 2002
Lukas Babicek
xbabice1@fi.muni.cz

Obsahuje funkce pro vyhledani nejlepsiho tahu, transp. tabulka,
historie, hash funkce
-------------------------------------------------------------------*/

#include<stdlib.h>
#include<time.h>
#include<stdio.h>
#include<limits.h>
#include "Search.h"
#include "Moves.h"
#include "Chessboard.h"
#include "Evaluator.h"



//globalni promenne

short onmove = WHITE;   //kdo je zrovna na tahu (z hlediska partie)
short nowonmove;        //kdo je zrovna na tahu (z hlediska alfa bety)


/*-------------------------------------------------------------------
void FreeMoves(PTMove move)

uvolni pamet pro vsechny prvky zretezeneho seznamu, ktery zacina
prvkem move

move .. 1. prvek zretezeneho seznamu
-------------------------------------------------------------------*/
void FreeMoves(PTMove move)
{

	PTMove pom;
  while (move!=NULL){
		pom = move;
	  move = move->next;
		free(pom);
	}
}


//nalezeni nejlepsiho tahu pomoci metody Alpha Beta
int FindBestMove(int hloubka, int alpha, int beta)
{
  PTMove mymoves, st_move, best;
	int pom;


  GenerateMoves();

	st_move = start_move;
  mymoves = st_move;
	best = NULL;

  while (mymoves) {
		ApplyMove(mymoves);
    pom = -AlphaBeta(hloubka-1, -beta, -alpha, 1); 
		UndoMove(mymoves);

		if (pom != -ALPHABETA_BAD_MOVE) {
		  if (pom > alpha) {
  			alpha = pom;
				best = mymoves;
			}
 		  if (pom>=beta) {
				bestmove = *best;
	  	  FreeMoves(st_move);
  	    return beta;
			}
		}
		mymoves = mymoves->next;
	} 

	if (!best){
		FreeMoves(st_move);
		return alpha;
	}

  bestmove = *best;
	FreeMoves(st_move);
	return alpha;
}

		
//metoda prohledavani stavoveho prostoru Alpha-Beta
int AlphaBeta(int hloubka, int alpha, int beta, short doquiesence)
{
  unsigned long i=0;
  PTMove mymoves, st_move;
  int pom, nmoves;


  if (hloubka == 0) {
		if (doquiesence)
		  return QuiesenceSearch(QUIESENCE_HLOUBKA, alpha, beta);
		else
			return StaticValue();
	}
    
	nmoves = GenerateMoves();

	st_move = mymoves = start_move;
  if (nmoves == KING_CHECKED) {
		FreeMoves(st_move);
    return ALPHABETA_BAD_MOVE;
	}

	while (mymoves != NULL){
    //poved tah
    ApplyMove(mymoves);
 	  pom = -AlphaBeta(hloubka-1, -beta, -alpha, doquiesence);
		//vrat tah
    UndoMove(mymoves);
		if (pom != -ALPHABETA_BAD_MOVE) {
		  if (pom > alpha) {
  			alpha = pom;
  		  if (pom>=beta) {
		  	  FreeMoves(st_move);
			    return beta;
				}
			}
		} 
		mymoves = mymoves -> next;
	}
	FreeMoves(st_move);
	return alpha;
}


//Dopocet do tiche pozice - jako alpha beta, ale bere pouze ty tahy, ktere vyrazne
//zmeni hodnotu materialu - brani, povyseni
int QuiesenceSearch(int hloubka, int alpha, int beta)
{
  unsigned long i=0;
  PTMove mymoves, st_move;
  int pom, nmoves;

  //NULL move
	pom = StaticValue();  
	if (pom>alpha){
		alpha = pom;
	}
	
	nmoves = GenerateQuiesenceMoves();

	if (nmoves == 0){
		return pom;
	}

	st_move = mymoves = start_move;
  if (nmoves == KING_CHECKED) {
		FreeMoves(st_move);
		return ALPHABETA_BAD_MOVE;
	}
  
	if (hloubka == 0) {
    FreeMoves(st_move);
		return pom;
	}

  while (mymoves != NULL){
    //poved tah
    ApplyMove(mymoves);
 	  pom = -QuiesenceSearch(hloubka-1, -beta, -alpha);
		//vrat tah
    UndoMove(mymoves);
		if (pom != -ALPHABETA_BAD_MOVE) {
		  if (pom > alpha) {
  			alpha = pom;
			}
  		if (pom>=beta) {
		    FreeMoves(st_move);
			  return beta;
			}
		} 
		mymoves = mymoves -> next;
	}
	FreeMoves(st_move);
	return alpha;
}
