/*-------------------------------------------------------------------
Winmanip.c
(c) 2002
Lukas Babicek
xbabice1@fi.muni.cz

Obsahuje funkce pro inicializaci oken, uzivatelsky interface, ...
-------------------------------------------------------------------*/
#include "Winmanip.h"
#include "Moves.h"
#include "Game.h"
#include "Chessboard.h"
#include "Search.h"
#include "Graphics.h"
#include <string.h>

//konstanty a globalni promenne

GLuint main_window=0;     // hlavni okno aplikace
GLuint board_window=0;    // okno s sachovnici
GLuint color_window=0;    // okno pro volbu barev
GLuint promotion_window=0;// okno pro volbu figurky pro povyseni

GLfloat main_window_width  = DEFAULT_MAIN_WINDOW_WIDTH;  //sirka hlavniho okna
GLfloat main_window_height = DEFAULT_MAIN_WINDOW_HEIGHT; //vyska hlavniho okna

GLfloat board_window_width  = DEFAULT_BOARD_WINDOW_WIDTH;  //sirka okna s sachovnici
GLfloat board_window_height = DEFAULT_BOARD_WINDOW_HEIGHT; //vyska okna s sachovnici

GLfloat color_window_width  = 400;  //sirka okna s barvami
GLfloat color_window_height = 450; //vyska okna s barvami

GLfloat promotion_window_width  = 200;  //sirka okna pro povyseni
GLfloat promotion_window_height = 50; //vyska okna pro povyseni

int color_choice; //pomocna promenna pro volbu barvy

int isdrag = 0;   //oretahuje uzivatel mysi nejakou figurku?
TDragfig dragfig; //informace o tazene figurce

char status_text[200]=""; //text, ktery se bude objevovat ve status baru

short promotion_figure=0; //figurka, kterou si hrac vybere v okne pro povyseni
short choosing_promotion = 0; //pomocna promenna, znaci, yda si hrac vybira figurku k povyseni


//prototypy
void MainWindowDisplay(void);


//definice funkci

/*-------------------------------------------------------------------
void ShowDeathPieces(void)

Vykresli do hlavniho okna vyhozene figurky
-------------------------------------------------------------------*/
void ShowDeathPieces(void){

  char message[]="Vyhozene figurky:";
  char *p;
	int i;
	float x, y, dx, dy;

  glRasterPos2f(board_window_width+(main_window_width-board_window_width)/2-50, 20);
	glColor3f(1.0,1.0,1.0); 
	for (p=message; *p;p++)
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,*p);

  dy = (main_window_height - 40) / 8;
  dx = (main_window_width - board_window_width -20) / 4;
	dx = (dx > dy) ? dy : dx;
	x = (board_window_width + 10);
	y = 40;

  for (i=0; i<black_death_number; i++) {

		switch (black_death_pieces[i]) {
  		case BLACK_PAWN:DrawPawn(x, y, dx, BLACK);break;
		  case BLACK_ROOK:DrawRook(x, y, dx, BLACK);break;
  		case BLACK_KNIGHT:DrawKnight(x, y, dx, BLACK);break;
		  case BLACK_BISHOP:DrawBishop(x, y, dx, BLACK);break;
  		case BLACK_QUEEN:DrawQueen(x, y, dx, BLACK);break;
  		case BLACK_KING:DrawKing(x, y, dx, BLACK);break;
		}

    x += dx;
		if ((i+1) % 4 == 0) {
     	x = (board_window_width + 10);
      y += dx;
		}
	}

  y = 40 + (main_window_height-40) / 2;
	x = (board_window_width + 10);

  for (i=0; i<white_death_number; i++) {

		switch (white_death_pieces[i]) {
  		case WHITE_PAWN:DrawPawn(x, y, dx, WHITE);break;
		  case WHITE_ROOK:DrawRook(x, y, dx, WHITE);break;
  		case WHITE_KNIGHT:DrawKnight(x, y, dx, WHITE);break;
		  case WHITE_BISHOP:DrawBishop(x, y, dx, WHITE);break;
  		case WHITE_QUEEN:DrawQueen(x, y, dx, WHITE);break;
  		case WHITE_KING:DrawKing(x, y, dx, WHITE);break;
		}

    x += dx;
		if ((i+1) % 4 == 0) {
     	x = (board_window_width + 10);
      y += dx;
		}
	}


}

/*-------------------------------------------------------------------
void DrawStatusText(void)

zobrazi status_text do status baru
-------------------------------------------------------------------*/
void DrawStatusText(void){

  int i;

	glColor3f(0.0,0.0,0.0); 
  glBegin(GL_POLYGON);
  glVertex2f(0, main_window_height-16);
  glVertex2f(main_window_width, main_window_height-16);
  glVertex2f(main_window_width, main_window_height);
  glVertex2f(0, main_window_height-16);
  glEnd();

	glColor3f(1.0,1.0,1.0); 
  glRasterPos2f(20, main_window_height-5);
  for (i=0; i<strlen(status_text);i++)
    glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12,status_text[i]);

}


/*-------------------------------------------------------------------
void SetStatusText(char *st)

nastavi status_text
-------------------------------------------------------------------*/
void SetStatusText(char *st)
{
	strcpy(status_text,st);
	glutSetWindow(main_window);
	glutPostRedisplay();
}


/*-------------------------------------------------------------------
void BoardWindowReshape(int width,int height)

Funkce pro odezvu na zmenu velikosti okna s sachovnici
-------------------------------------------------------------------*/

void BoardWindowReshape(int width,int height)
{
  glutSetWindow(board_window);
  board_window_width=width;
  board_window_height=height;
  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, width, height, 0);
}

/*-------------------------------------------------------------------
void BoardWindowDisplay(void)

Funkce pro odezvu na vykresleni okna s sachovnici
-------------------------------------------------------------------*/

void BoardWindowDisplay(void)
{
  glutSetWindow(board_window);
  glViewport(0,0,board_window_width, board_window_height);
  //smazani okna
  glClearColor(0, 0, 0,0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  //vykresleni sachovnice
  DrawChessboard(board_window_width);
  //pozadame o prekresleni
  glutPostRedisplay();
  //vymenime buffery
  glutSwapBuffers();
}

/*-------------------------------------------------------------------
void BoardMouse(int button, int state, int x, int y)

Funkce pro odezvu na stisk tlacitka mysi v okne s sachovnici
Zde je take osetren vyber figurky pro tazeni
-------------------------------------------------------------------*/

void BoardMouse(int button, int state, int x, int y)
{
	double dx;
	int i,j,pos;
	short fig,col;

  if (choosing_promotion || end) return;
	
	if (button==GLUT_LEFT_BUTTON && state==GLUT_DOWN) {
		if (x>BOARD_BORDER_LINE_WIDTH + BOARD_GAP &&
			  x<board_window_width-BOARD_GAP &&
			  y>BOARD_BORDER_LINE_WIDTH &&
			  y<board_window_height-BOARD_GAP-BOARD_BORDER_LINE_WIDTH) {
      
			dx = (board_window_width-2*BOARD_BORDER_LINE_WIDTH-BOARD_GAP)/8;
			i = (double)(x-BOARD_BORDER_LINE_WIDTH-BOARD_GAP) / dx;
			j = (double)(y-BOARD_BORDER_LINE_WIDTH) / dx;
			i += 2;
			j += 2;
			pos = i+j*12;
      fig = chessboard[pos];
			col = (fig>0)? WHITE:BLACK;
			if (col == WHITE && white_player == PLAYER_COMPUTER)
				return;
			if (col == BLACK && black_player == PLAYER_COMPUTER)
				return;
			if (fig!= EMPTY && col==onmove) {
				dragfig.x = x-dx/2;
				dragfig.y = y-dx/2;
				dragfig.oldpos = pos;
				dragfig.figure = fig;
        isdrag = 1;
			}
		}
	}

  if (button==GLUT_LEFT_BUTTON && state==GLUT_UP && isdrag) {
		isdrag = 0;
    if (x>BOARD_BORDER_LINE_WIDTH + BOARD_GAP &&
			  x<board_window_width-BOARD_GAP &&
			  y>BOARD_BORDER_LINE_WIDTH &&
				y<board_window_height-BOARD_GAP-BOARD_BORDER_LINE_WIDTH){
  		dx = (board_window_width-2*BOARD_BORDER_LINE_WIDTH-BOARD_GAP)/8;
	  	i = (double)(x-BOARD_BORDER_LINE_WIDTH-BOARD_GAP) / dx;
		  j = (double)(y-BOARD_BORDER_LINE_WIDTH) / dx;
			i += 2;
			j += 2;
			pos = i+j*12;
      dragfig.x = pos;
      if (ControlMove(dragfig.oldpos, pos)) {  //zde dochazi ke kontrole, zda je tah legalni
  		  onmove = -onmove;
        if (onmove == WHITE) {
				  	SetStatusText("Na tahu je bily.");
				}
        else  {
           	SetStatusText("Na tahu je cerny.");
        }
        BoardWindowDisplay();
        MainWindowDisplay();
			}
			else if (!choosing_promotion)
		  	SetStatusText("Ilegalni tah!");

		}

	}

}
	     

/*-------------------------------------------------------------------
void BoardMotion(int x, int y)

Funkce pro odezvu na pohyb mysi v okne s sachovnici
Je zde osetreno tayeni figurky v okne
-------------------------------------------------------------------*/

void BoardMotion(int x, int y)
{
  double dx;
  
	if (isdrag) {
		dx = (board_window_width-2*BOARD_BORDER_LINE_WIDTH-BOARD_GAP)/8;
		dragfig.x= x = x - dx/2;
		dragfig.y= y = y - dx/2;
		glutPostRedisplay();
	}

}
/*-------------------------------------------------------------------
void ColorWindowReshape(int width,int height)

Funkce pro odezvu na zmenu velikosti okna pro volbu barev
-------------------------------------------------------------------*/

void ColorWindowReshape(int width,int height)
{
  glutSetWindow(color_window);
  glViewport(0,0,width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, width, height, 0);
}

/*-------------------------------------------------------------------
void ColorWindowDisplay(void)

Funkce pro odezvu na vykresleni okna pro volbu barev
-------------------------------------------------------------------*/

void ColorWindowDisplay(void)
{
  int i=0,j=0;
  double r,g,b;

  glutSetWindow(color_window);
  glViewport(0,0,color_window_width, color_window_height);
  //smazani okna
  glClearColor(0, 0, 0,0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  //namalujeme barvicky
  for (r=0;r<=1.0;r+=0.33) {
    for (g=0;g<=1.0;g+=0.33) {
      for (b=0;b<=1.0;b+=0.33){
        glColor3f(r,g,b);
        glRectd(i,j,i+50,j+50);
        i+=50;
        if (i>=400){
          i=0;
          j+=50;
        }
      }
    }
  }
  i=0;
  for (r=0.125;r<=1.02;r+=0.125) {
    glColor3f(r,r,r);
    glRectd(i,400,i+50,450);
    i+=50;
  }

  //pozadani o prekresleni
  glutPostRedisplay();
  //vymenime buffery
  glutSwapBuffers();
}

/*-------------------------------------------------------------------
void ColorMouseFunc(int button, int state, int x, int y)

Funkce pro odezvu na stisk tlacitka mysi v okne pro volbu barev
-------------------------------------------------------------------*/
void ColorMouseFunc(int button, int state, int x, int y)
{
  double r,g,b;
  int position;

  if (state == GLUT_DOWN) {
    position=(x/50 + (y/50)*8);
    if (position < 64) {
      b=0.3333 * (position % 4);
      g=0.3333 * ((position/4) % 4);
      r=0.3333 * (position/16);
    }
    else
      r=g=b=0.125 * (position-63);
  }
  SetColor(color_choice,r,g,b);
  glutDestroyWindow(color_window);
  color_window=0;
  glutSetWindow(board_window);
  glutPostRedisplay();
}

/*-------------------------------------------------------------------
void ColorMenu(int value)

Funkce pro odezvu volbu v menu okna s barvami
-------------------------------------------------------------------*/
void ColorMenu(int value){
  glutDestroyWindow(color_window);
  color_window=0;
}


/*-------------------------------------------------------------------
void CreateColorWindow(void)

Vytvori okno pro vyber barev (pokud neexistuje);
-------------------------------------------------------------------*/
void CreateColorWindow(void){
  glutSetWindow(main_window);
  if (color_window==0){
    color_window = glutCreateSubWindow(main_window, 0, 0, color_window_width, color_window_height);
    glutDisplayFunc(ColorWindowDisplay);
    glutReshapeFunc(ColorWindowReshape);
    glutMouseFunc(ColorMouseFunc);
    glutCreateMenu(ColorMenu);
    glutAddMenuEntry("Zavrit", 0);
    glutAttachMenu(GLUT_RIGHT_BUTTON);
  }
  glutSetWindow(board_window);
  glutPushWindow();

}


/*-------------------------------------------------------------------
void PromotionWindowReshape(int width,int height)

Funkce pro odezvu na zmenu velikosti okna pro povyseni
-------------------------------------------------------------------*/

void PromotionWindowReshape(int width,int height)
{
  glutSetWindow(promotion_window);
  glViewport(0,0,width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, width, height, 0);
}

/*-------------------------------------------------------------------
void PromotionWindowDisplay(void)

Funkce pro odezvu na vykresleni okna pro povyseni
-------------------------------------------------------------------*/

void PromotionWindowDisplay(void)
{

  glutSetWindow(promotion_window);
  glViewport(0,0,promotion_window_width, promotion_window_height);
  glClearColor(0.3, 0.3, 0.3,0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  
	if (onmove==WHITE) {
    DrawQueen(0,0,50,WHITE);
    DrawRook(50,0,50,WHITE);
    DrawBishop(100,0,50,WHITE);
    DrawKnight(150,0,50,WHITE);
	}
  else{
    DrawQueen(0,0,50,BLACK);
    DrawRook(50,0,50,BLACK);
    DrawBishop(100,0,50,BLACK);
    DrawKnight(150,0,50,BLACK);
	}		
	
	//pozadani o prekresleni
  glutPostRedisplay();
  //vymenime buffery
  glutSwapBuffers();
}

/*-------------------------------------------------------------------
void PromotionMouseFunc(int button, int state, int x, int y)

Funkce pro odezvu na stisk tlacitka mysi v okne pro povyseni
-------------------------------------------------------------------*/
void PromotionMouseFunc(int button, int state, int x, int y)
{

  if (state == GLUT_DOWN) {
      if (x<50) promotion_figure = QUEEN;
      else if (x<100) promotion_figure = ROOK;
      else if (x<150) promotion_figure = BISHOP;
      else promotion_figure = KNIGHT;
		glutDestroyWindow(promotion_window);
    promotion_window=0;

  if (ControlMove(dragfig.oldpos, dragfig.x)) {  //zde dochazi ke kontrole, zda je tah legalni
  		  onmove = -onmove;
        if (onmove == WHITE) {
				  	SetStatusText("Na tahu je bily.");
				}
        else  {
           	SetStatusText("Na tahu je cerny.");
        }
	}
	else
	 	SetStatusText("Ilegalni tah!");
	
  BoardWindowDisplay();
  MainWindowDisplay();
	}
}



/*-------------------------------------------------------------------
void CreatePromotionWindow(void)

Vytvori okno pro vyber figurky k povyseni
-------------------------------------------------------------------*/
void CreatePromotionWindow(void){
  glutSetWindow(main_window);
  if (promotion_window==0){
    if (onmove == WHITE)
      promotion_window = glutCreateSubWindow(main_window, 0, 0, promotion_window_width, promotion_window_height);
		else
			promotion_window = glutCreateSubWindow(main_window, 0, board_window_height-50, promotion_window_width, promotion_window_height);
    glutDisplayFunc(PromotionWindowDisplay);
    glutReshapeFunc(PromotionWindowReshape);
    glutMouseFunc(PromotionMouseFunc);
  }
  glutSetWindow(board_window);
  glutPushWindow();
}


/*-------------------------------------------------------------------
void BoardMenu(int value)

Funkce pro odezvu na volbu v menu okna s sachovnici
-------------------------------------------------------------------*/
void BoardMenu(int value){
  color_choice=value;
  CreateColorWindow();
}

/*-------------------------------------------------------------------
void BoardIdle(void)

Funkce pro odezvu na nicnedelani v okne s sachovnici
-------------------------------------------------------------------*/
void BoardIdle(void){

  CheckEnding();

  
	if (!end && !choosing_promotion) {
	  if (black_player == PLAYER_COMPUTER){
  		DoBlackMove();
		} 
    if (white_player == PLAYER_COMPUTER){
		  DoWhiteMove();
		}
	}

}

/*-------------------------------------------------------------------
void MainWindowReshape(int width,int height)

Funkce pro odezvu na zmenu velikosti hlavniho okna
-------------------------------------------------------------------*/

void MainWindowReshape(int width,int height)
{
  main_window_width=width;
  main_window_height=height;
  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, width, height, 0);
  //pri zmene rozmeru hlavniho okna se zmeni i okno s sachovnici
  glutSetWindow(board_window);
  glutPositionWindow(0,0);
  glutReshapeWindow(height-20,height-20);
}

/*-------------------------------------------------------------------
void MainWindowDisplay(void)

Funkce pro odezvu na vykresleni hlavniho okna
-------------------------------------------------------------------*/

void MainWindowDisplay(void)
{
  glutSetWindow(main_window);
  glViewport(0,0,main_window_width, main_window_height);
  //smazani okna
  glClearColor(0, 0, 0,0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  ShowDeathPieces(); //zobrazeni vyhozenych figurek
  DrawStatusText(); //zobrazeni status textu
  //pozadame o prekresleni
  glutPostRedisplay();
  //vymenime buffery
  glutSwapBuffers();
}

/*-------------------------------------------------------------------
void MainMenu(int value)

Funkce pro odezvu volbu v menu okna s sachovnici
-------------------------------------------------------------------*/
void MainMenu(int value){
  switch(value){
    case 0:
      white_player = PLAYER_HUMAN;   
      black_player = PLAYER_COMPUTER;
			InitGame();
  		glutSetWindow(main_window);
			glutPostRedisplay();
      break;
    case 1:
      white_player = PLAYER_COMPUTER;   
      black_player = PLAYER_HUMAN;
			InitGame();
  		glutSetWindow(main_window);
			glutPostRedisplay();
      break;
    case 2:
      white_player = PLAYER_HUMAN;   
      black_player = PLAYER_HUMAN;
			InitGame();
  		glutSetWindow(main_window);
			glutPostRedisplay();
      break;
    case 3:
      white_player = PLAYER_COMPUTER;   
      black_player = PLAYER_COMPUTER;
			InitGame();
  		glutSetWindow(main_window);
			glutPostRedisplay();
      break;
    case 10:
      glutDestroyWindow(board_window);
      glutDestroyWindow(main_window);
      exit(0); 
      break;
  }
}

/*-------------------------------------------------------------------
int PrepareWindows(int argc, char **argv)

Vytvori okna a priradi jim funkce pro odezvu na udalosti
-------------------------------------------------------------------*/
int PrepareWindows(int argc, char **argv)
{

  //inicializace vseho potrebneho
	
	//inicializace OpenGL
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);

  glutInitWindowSize(main_window_width, main_window_height);
  glutInitWindowPosition(0, 0);
  glutInit(&argc, argv);

  //vytvoreni hlavniho okna
  main_window = glutCreateWindow("Chess");
  glutDisplayFunc(MainWindowDisplay);
  glutReshapeFunc(MainWindowReshape);
  glutCreateMenu(MainMenu);
  glutAddMenuEntry("Nova hra: bily - clovek, cerny - pocitac", 0);
  glutAddMenuEntry("Nova hra: bily - pocitac, cerny - clovek", 1);
  glutAddMenuEntry("Nova hra: bily - clovek, cerny - clovek", 2);
  glutAddMenuEntry("Nova hra: bily - pocitac, cerny - pocitac", 3);
  glutAddMenuEntry("Konec", 10);
  glutAttachMenu(GLUT_RIGHT_BUTTON);

  //vytvoreni okna s sachovnici
  board_window = glutCreateSubWindow(main_window, 0, 0, board_window_width, board_window_height);
  glutDisplayFunc(BoardWindowDisplay);
  glutReshapeFunc(BoardWindowReshape);
  glutMouseFunc(BoardMouse);
	glutMotionFunc(BoardMotion);
	glutIdleFunc(BoardIdle);
  glutCreateMenu(BoardMenu);
  glutAddMenuEntry("Zvolit barvu bileho pole", 0);
  glutAddMenuEntry("Zvolit barvu cerneho pole", 1);
  glutAddMenuEntry("Zvolit barvu bilych figurek ", 2);
  glutAddMenuEntry("Zvolit barvu cernych figurek ", 3);
  glutAddMenuEntry("Zvolit barvu okraje bilych figurek ", 4);
  glutAddMenuEntry("Zvolit barvu okraje cernych figurek ", 5);
  glutAddMenuEntry("Zvolit barvu okraje sachovnice", 6);
  glutAddMenuEntry("Zvolit barvu popisku sachovnice", 7);

  glutAttachMenu(GLUT_RIGHT_BUTTON);

  //maximalizovani hlavniho okna (SYSTEMOVE ZAVISLE!!)
  glutSetWindow(main_window);
//  glutReshapeWindow(GetSystemMetrics(SM_CXFULLSCREEN), GetSystemMetrics(SM_CYFULLSCREEN));

   //hlavni smycka
	InitGame();
  glutMainLoop();

  return 0;
}